! ================================================================================================================================
! MODULE       : stomate_lpj
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Main entry point for daily processes in STOMATE and LPJ (phenology, 
!! allocation, npp_calc, kill, turn, light, establish, crown, cover, lcchange)
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S) : None
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_lpj

  ! modules used:

  USE function_library,   ONLY : check_biomass_sync, wood_to_qmdia, wood_to_qmheight, &
                                 biomass_to_lai
  USE ioipsl_para
  USE xios_orchidee
  USE grid
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE structures
  USE lpj_constraints
  USE lpj_pftinout
  USE lpj_kill
  USE lpj_crown
  USE lpj_fire
  USE lpj_gap
  USE lpj_light
  USE lpj_establish
  USE lpj_cover
  USE stomate_kill
  USE stomate_mark_kill
  USE stomate_prescribe
  USE stomate_phenology
  USE stomate_growth_fun_all
  USE stomate_growth_res_lim
  USE stomate_stand_structure
  USE stomate_turnover
  USE stomate_litter
  USE stomate_laieff, ONLY : find_lai_per_level, calculate_z_level_photo,&
                             combine_lai_levels, fitting_laieff
  USE stomate_soilcarbon
  USE stomate_vmax
  USE stomate_assimtemp
  USE sapiens_lcchange
  USE sapiens_forestry
  USE sapiens_kill
  USE sapiens_agriculture
  USE sapiens_product_use

!  USE Orch_Write_field_p


  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC StomateLpj,StomateLpj_clear

  !+++CHECK+++
  ! firstcall is not used in stomate_lpj -> redundant?
  LOGICAL, SAVE                         :: firstcall = .TRUE.             !! first call
 !$OMP THREADPRIVATE(firstcall)
  !+++++++++++

CONTAINS

!! ================================================================================================================================
!! SUBROUTINE   : StomateLpj_clear
!!
!>\BRIEF        Re-initialisation of variable
!!
!! DESCRIPTION  : This subroutine reinitializes variables. To be used if we want to relaunch 
!! ORCHIDEE but the routine is not used in current version.
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S): None
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE StomateLpj_clear

    CALL prescribe_clear
    CALL phenology_clear
    CALL growth_res_lim_clear
    CALL growth_fun_all_clear
    CALL turnover_clear
    CALL soilcarbon_clear
    CALL constraints_clear
    CALL establish_clear
    CALL fire_clear
    CALL gap_clear
    CALL light_clear
    CALL pftinout_clear

  END SUBROUTINE StomateLpj_clear


!! ================================================================================================================================
!! SUBROUTINE   : StomateLPJ
!!
!>\BRIEF        Main entry point for daily processes in STOMATE and LPJ, structures the call sequence 
!!              to the different processes such as dispersion, establishment, competition and mortality of PFT's.
!! 
!! DESCRIPTION  : This routine is the main entry point to all processes calculated on a 
!! daily time step. Is mainly devoted to call the different STOMATE and LPJ routines 
!! depending of the ok_dgvm (is dynamic veg used) and lpj_constant_mortality (is background mortality used).
!! It also prepares the cumulative 
!! fluxes or pools (e.g TOTAL_M TOTAL_BM_LITTER etc...)
!!
!! This routine makes frequent use of "weekly", "monthly" and "long term" variables. Caution is needed because
!! by default "weekly" denotes 7 days, by default "monthly" denotes 20 days and by default "Long term" denotes
!! 3 years. dtslow refers to 24 hours (1 day).
!!
!!
!! RECENT CHANGE(S) : None
!! 
!! MAIN OUTPUT VARIABLE(S): All variables related to stomate and required for LPJ dynamic vegetation mode.
!!
!! REFERENCE(S) : 
!! - Krinner, G., N. Viovy, N. de Noblet-Ducoudré, J. Ogeé, J. Polcher, P. Friedlingstein, P. Ciais, S. Sitch, 
!! and I. C. Prentice. 2005. A dynamic global vegetation model for studies of the coupled atmosphere-biosphere 
!! system. Global Biogeochemical Cycles 19:GB1015, doi:1010.1029/2003GB002199.
!! - Sitch, S., B. Smith, I. C. Prentice, A. Arneth, A. Bondeau, W. Cramer, J. O. Kaplan, S. Levis, W. Lucht, 
!! M. T. Sykes, K. Thonicke, and S. Venevsky. 2003. Evaluation of ecosystem dynamics, plant geography and 
!! terrestrial carbon cycling in the LPJ dynamic global vegetation model. Global Change Biology 9:161-185.
!!
!! FLOWCHART    : Update with existing flowchart from N Viovy (Jan 19, 2012)
!! \n
!_ ================================================================================================================================
 
  SUBROUTINE StomateLpj (npts, dt_days, EndOfYear, EndOfMonth, neighbours, &
       resolution, clay, herbivores, tsurf_daily, tsoil_daily, &
       t2m_daily, t2m_min_daily, litterhum_daily, soilhum_daily, maxmoiavail_lastyear, &
       minmoiavail_lastyear, gdd0_lastyear, precip_lastyear, moiavail_month, moiavail_week, &
       moiavail_growingseason, tlong_ref, t2m_longterm, t2m_month, t2m_week, tsoil_month, &
       soilhum_month, gdd_m5_dormance, gdd_from_growthinit, gdd_midwinter, ncd_dormance, &
       ngd_minus5, turnover_longterm, gpp_daily, gpp_week, time_lowgpp, &
       time_hum_min, maxfpc_lastyear, resp_maint_part, PFTpresent, age, &
       fireindex, firelitter, leaf_age, leaf_frac, biomass, &
       ind, adapted, regenerate, senescence,  senescence_days, &
       when_growthinit, litterpart, litter, dead_leaves, carbon, &
       black_carbon, lignin_struc, lignin_wood, veget_max, veget, npp_longterm, &
       lm_lastyearmax, veget_lastlight, everywhere, need_adjacent, RIP_time, &
       rprof, npp_daily, turnover_daily, turnover_time, control_moist, &
       control_temp, soilcarbon_input, co2_to_bm, co2_fire, resp_hetero, &
       resp_maint, resp_growth, deadleaf_cover, vcmax, t_photo_min, &
       t_photo_opt, t_photo_max, t_photo_stress, bm_to_litter, prod_s, &
       prod_m, prod_l, flux_s, flux_m, flux_l, &
       veget_max_new, cflux_prod_s, cflux_prod_m, cflux_prod_l, carb_mass_total, & 
       fpc_max, age_stand, rotation_n, last_cut, forest_managed, &
       forest_managed_lastyear, MatrixA, rue_longterm, lab_fac, circ_class_n, &
       circ_class_biomass, KF, circ_class_dist, harvest_pool_bound, harvest_pool, &
       harvest_type, harvest_cut, harvest_area, mai, pai, &
       previous_wood_volume, mai_count, bm_sync, coppice_dens, lai_per_level, &
       laieff_fit, orphan_flux, vir_moiavail_growingseason, litter_demand, wstress_season, &
       wstress_month, vir_wstress_fac, tau_eff_leaf, &
       tau_eff_sap, tau_eff_root, MatrixV, VectorB, VectorU, &
       c_buried, st_dist, k_latosa_adapt)
    
  !! 0. Variable and parameter declaration

    !! 0.1 input

    INTEGER(i_std), INTENT(in)                       :: npts                    !! Domain size (unitless)
    REAL(r_std), INTENT(in)                          :: dt_days                 !! Time step of Stomate (days)
    INTEGER(i_std), DIMENSION(:,:), INTENT(in)       :: neighbours              !! Indices of the 8 neighbours of each grid 
                                                                                !! point [1=N, 2=NE, 3=E, 4=SE,
                                                                                !!  5=S, 6=SW, 7=W, 8=NW]
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: resolution              !! Resolution at each grid point (m)  
                                                                                !! [1=E-W, 2=N-S] 
    REAL(r_std), DIMENSION(:), INTENT(in)            :: clay                    !! Clay fraction (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: herbivores              !! Time constant of probability of a leaf to 
                                                                                !! be eaten by a herbivore (days) 
    REAL(r_std), DIMENSION(:), INTENT(in)            :: tsurf_daily             !! Daily surface temperatures (K)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tsoil_daily             !! Daily soil temperatures (K)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: t2m_daily               !! Daily 2 meter temperatures (K)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: t2m_min_daily           !! Daily minimum 2 meter temperatures (K)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: litterhum_daily         !! Daily litter humidity (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: soilhum_daily           !! Daily soil humidity (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: maxmoiavail_lastyear    !! Last year's maximum moisture availability 
                                                                                !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: minmoiavail_lastyear    !! Last year's minimum moisture availability 
                                                                                !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: gdd0_lastyear           !! Last year's GDD0 (K)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: precip_lastyear         !! Lastyear's precipitation 
                                                                                !! @tex $(mm year^{-1})$ @endtex
                                                                                !! to determine if establishment possible
    REAL(r_std), DIMENSION(:), INTENT(in)            :: tlong_ref               !! "Long term" 2 meter reference 
                                                                                !! temperatures (K)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: t2m_longterm            !! "longterm" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: t2m_month               !! "Monthly" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: t2m_week                !! "Weekly" 2-meter temperatures (K)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tsoil_month             !! "Monthly" soil temperatures (K)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: soilhum_month           !! "Monthly" soil humidity
                                                                                !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(in)      :: turnover_longterm       !! "Long term" turnover rate  
                                                                                !! @tex $(gC m^{-2} year^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: time_lowgpp             !! Duration of dormance period with low gpp
                                                                                !! (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: maxfpc_lastyear         !! Last year's maximum foliage projected
                                                                                !! coverage for each natural PFT,
                                                                                !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)        :: resp_maint_part         !! Maintenance respiration of different 
                                                                                !! plant parts  
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: fpc_max                 !! "Maximal" coverage fraction of a PFT
                                                                                !! (LAI -> infinity) on ground  
                                                                                !! @tex $(m^2 m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_root            !! Effective root turnover time that accounts
                                                                                !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_sap             !! Effective sapwood turnover time that accounts
                                                                                !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_leaf            !! Effective leaf turnover time that accounts
                                                                                !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: vir_wstress_fac         !! Virtual water stress factor, based on hum_rel_daily
                                                                                !! (unitless, 0-1)
    LOGICAL, INTENT(in)                              :: EndOfYear               !! Flag set on the last day of the year used 
                                                                                !! to update "yearly variables". This 
                                                                                !! variable must be .TRUE. once a year
    LOGICAL, INTENT(in)                              :: EndOfMonth              !! Flag set at end of each month to update 
                                                                                !! monthly variable 
    REAL(r_std), DIMENSION(:), INTENT(in)            :: circ_class_dist         !! The probability distribution of trees 
                                                                                !! in a circ class in case of a 
                                                                                !! redistribution (unitless).
    REAL(r_std), DIMENSION(:), INTENT(in)            :: st_dist                 !! The probability distribution of trees 
                                                                                !! removed from a circ class in case of  
                                                                                !! self thinning (unitless).
    REAL(r_std), DIMENSION(0:), INTENT(in)           :: harvest_pool_bound      !! The boundaries of the diameter classes
                                                                                !! in the wood harvest pools
                                                                                !! @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: vir_moiavail_growingseason !! Virtual mean growingseason moisture 
                                                                                !! availability (0 to 1, unitless)
    REAL(r_std),DIMENSION(:,:), INTENT(in)           :: veget                   !! Fraction of vegetation type including 
                                                                                !! non-biological fraction (unitless) 


  !! 0.2 Output variables
    
    REAL(r_std), DIMENSION(:,:), INTENT(out)         :: npp_daily               !! Net primary productivity 
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)         :: co2_to_bm               !! CO2 taken up from atmosphere when 
                                                                                !! introducing a new PFT (introduced for 
                                                                                !! carbon balance closure) 
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)         :: co2_fire                !! Carbon emitted into the atmosphere by 
                                                                                !! fire (living and dead biomass)  
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)         :: resp_maint              !! Maintenance respiration  
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(out)         :: resp_growth             !! Growth respiration  
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    TYPE(laieff_type),DIMENSION (:,:,:),INTENT(out)  :: laieff_fit              !! Fitted parameters for the effective LAI
    REAL(r_std), DIMENSION(:,:), INTENT(out)         :: vcmax                   !! Maximum rate of carboxylation 
    REAL(r_std), DIMENSION(:,:), INTENT(out)         :: t_photo_min             !! Minimum temperature for photosynthesis 
                                                                                !! (C) This is not K because it is a scalar.
    REAL(r_std), DIMENSION(:,:), INTENT(out)         :: t_photo_opt             !! Optimum temperature for photosynthesis 
                                                                                !! (C) This is not K because it is a scalar.
    REAL(r_std), DIMENSION(:,:), INTENT(out)         :: t_photo_max             !! Maximum temperature for photosynthesis 
                                                                                !! (C) This is not K because it is a scalar.
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(out)     :: bm_to_litter            !! Conversion of biomass to litter
                                                                                !! trees whose circumference is smaller than 
                                                                                !! sigma don't grow much)
    REAL(r_std), DIMENSION(:,:), INTENT(out)         :: lab_fac                 !! Activity of labile pool factor (??units??)
    REAL(r_std), DIMENSION(:,:,:), INTENT(out)       :: lai_per_level           !! This is the LAI per vertical level
                                                                                !! @tex $(m^{2} m^{-2})$


    !! 0.3 Modified variables
    
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: turnover_daily          !! Turnover rates 
                                                                                !! @tex $(gC m^{-2} year^{-1})$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(inout)         :: deadleaf_cover          !! Fraction of soil covered by dead leaves 
                                                                                !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: resp_hetero             !! Heterotrophic respiration
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gpp_daily               !! Daily gross primary productivity  
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gpp_week                !! Mean weekly gross primary productivity 
                                                                                !! @tex $(gC m^{-2} day^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: time_hum_min            !! Time elapsed since strongest moisture 
                                                                                !! availability (days) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gdd_midwinter           !! Growing degree days (K), since midwinter 
                                                                                !! (for phenology) - this is written to the
                                                                                !!  history files
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gdd_m5_dormance         !! Growing degree days (K), threshold -5 deg 
                                                                                !! C (for phenology) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: gdd_from_growthinit     !! growing degree days, since growthinit 
                                                                                !! for crops
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ncd_dormance            !! Number of chilling days (days), since 
                                                                                !! leaves were lost (for phenology)    
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: control_moist           !! Moisture control of heterotrophic 
                                                                                !! respiration (0 to 1, unitless) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: control_temp            !! Temperature control of heterotrophic 
                                                                                !! respiration, above and below 
                                                                                !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: rue_longterm            !! Longterm radiation use efficiency 
                                                                                !! (??units??) 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: soilcarbon_input        !! Quantity of carbon going into carbon 
                                                                                !! pools from litter decomposition  
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: rprof                   !! Prescribed root depth (m) 
    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: PFTpresent              !! Tab indicating which PFTs are present in 
                                                                                !! each pixel 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: age                     !! Age (years)    
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: fireindex               !! Probability of fire (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: firelitter              !! Longer term litter above the ground that 
                                                                                !! can be burned, @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: t_photo_stress          !! Temperature stress for CEXCHANGE 
                                                                                !! photosynthesis (0-1)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_age                !! Leaf age (days)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_frac               !! Fraction of leaves in leaf age class, 
                                                                                !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: biomass                 !! Stand level biomass @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ind                     !! Density of individuals 
                                                                                !! @tex $(m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: adapted                 !! Adaptation of PFT (killed if too cold) 
                                                                                !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: regenerate              !! "Fitness": Winter sufficiently cold for 
                                                                                !! PFT regeneration ? (0 to 1, unitless) 
    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: senescence              !! Flag for setting senescence stage (only 
                                                                                !! for deciduous trees)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: senescence_days         !! days since the beginning of senescence
                                                                                !! (days)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: when_growthinit         !! How many days ago was the beginning of 
                                                                                !! the growing season (days)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: litterpart              !! Fraction of litter above the ground 
                                                                                !! belonging to different PFTs
                                                                                !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: litter                  !! Metabolic and structural litter, above 
                                                                                !! and below ground 
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: dead_leaves             !! Dead leaves on ground, per PFT, metabolic 
                                                                                !! and structural,  
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: carbon                  !! Carbon pool: active, slow, or passive, 
                                                                                !! @tex $(gC m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(:), INTENT(inout)         :: black_carbon            !! Black carbon on the ground 
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: lignin_struc            !! Ratio of Lignin/Carbon in structural 
                                                                                !! litter, above and below ground,  
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: lignin_wood             !! Ratio of Lignin/Carbon in woody 
                                                                                !! litter, above and below ground,  
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: veget_max               !! "Maximal" coverage fraction of a PFT (LAI 
                                                                                !! -> infinity) on ground. Includes
                                                                                !! the nobio fraction, so may sum to
                                                                                !! less than unity.
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: npp_longterm            !! "Long term" mean yearly primary 
                                                                                !! productivity 
                                                                                !! @tex $(gC m^{-2} year^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: lm_lastyearmax          !! Last year's maximum leaf mass, for each 
                                                                                !! PFT @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: veget_lastlight         !! Vegetation fractions (on ground) after 
                                                                                !! last light competition  
                                                                                !! @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: everywhere              !! Is the PFT everywhere in the grid box or 
                                                                                !! very localized (after its introduction) 
                                                                                !! (unitless) 
    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: need_adjacent           !! In order for this PFT to be introduced, 
                                                                                !! does it have to be present in an 
                                                                                !! adjacent grid box? 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: RIP_time                !! How much time ago was the PFT eliminated 
                                                                                !! for the last time (y) 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: turnover_time           !! Turnover_time of leaves for grasses 
                                                                                !! (days)
    REAL(r_std), DIMENSION(:,:),INTENT(inout)        :: veget_max_new           !! New "maximal" coverage fraction of a PFT 
                                                                                !! (LAI -> infinity)  Includes
                                                                                !! the nobio fraction, so may sum to
                                                                                !! less than unity. (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ngd_minus5              !! Number of growing days (days), threshold 
                                                                                !! -5 deg C (for phenology)    
    REAL(r_std), DIMENSION(:), INTENT(inout)         :: cflux_prod_s            !! C-released during first years (short term) 
                                                                                !! following land cover change 
                                                                                !! @tex ($gC year^{-1}$) @endtex 
    REAL(r_std), DIMENSION(:), INTENT(inout)         :: cflux_prod_m            !! Total annual release from decomposition of 
                                                                                !! the medium-lived product pool 
                                                                                !! @tex ($gC year^{-1}$) @endtex
    REAL(r_std), DIMENSION(:), INTENT(inout)         :: cflux_prod_l            !! Total annual release from decomposition of 
                                                                                !! the long-lived product pool 
                                                                                !! @tex ($gC year^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,0:), INTENT(inout)      :: prod_s                  !! Short-lived product pool after the annual 
                                                                                !! release of each compartment (short + 1 : 
                                                                                !! input from year of land cover change) 
                                                                                !! @tex ($gC$) @endtex    
    REAL(r_std), DIMENSION(:,0:), INTENT(inout)      :: prod_m                  !! Medium-lived product pool after the annual
                                                                                !! release of each compartment (medium + 1 : 
                                                                                !! input from year of land cover change) 
                                                                                !! @tex ($gC$) @endtex 
    REAL(r_std), DIMENSION(:,0:), INTENT(inout)      :: prod_l                  !! long-lived product pool after the annual 
                                                                                !! release of each compartment (long + 1 : 
                                                                                !! input from year of land cover change) 
                                                                                !! @tex ($gC$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: flux_s                  !! Annual release from the short-lived product 
                                                                                !! pool @tex ($gC) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: flux_m                  !! Annual release from the medium-lived product  
                                                                                !! pool compartments @tex ($gC) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: flux_l                  !! Annual release from the long-lived product
                                                                                !! pool @tex ($gC) @endtex 
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: circ_class_biomass      !! Biomass of the componets of the model  
                                                                                !! tree within a circumference
                                                                                !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: circ_class_n            !! Number of individuals in each circ class
                                                                                !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(inout)         :: carb_mass_total         !! Carbon Mass total (soil, litter, veg) 
                                                                                !! @tex $(gC m^{-2})$ @endtex  
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: MatrixA                 !! Matrix containing the fluxes  
                                                                                !! between the carbon pools
                                                                                !! per sechiba time step 
                                                                                !! @tex $(gC.m^2.day^{-1})$ @endtex
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: age_stand               !! Age of stand (years)
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: rotation_n              !! Rotation number (number of rotation 
                                                                                !! since pft is managed)
    INTEGER(i_std), DIMENSION(:,:), INTENT(inout)    :: last_cut                !! Years since last thinning (years)
    INTEGER(i_std), DIMENSION (:,:), INTENT(inout)   :: forest_managed          !! forest management flag (is the forest 
                                                                                !! being managed?)
    INTEGER(i_std), DIMENSION (:,:), INTENT(inout)   :: forest_managed_lastyear !! forest management flag for the 
                                                                                !! previous year
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: KF                      !! Scaling factor to convert sapwood mass
                                                                                !! into leaf mass (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: k_latosa_adapt          !! Leaf to sapwood area adapted for long 
                                                                                !! term water stress (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: moiavail_month          !! "Monthly" moisture availability (0 to 1, 
                                                                                !! unitless) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: moiavail_week           !! "Weekly" moisture availability 
                                                                                !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: moiavail_growingseason  !! Mean growingseason moisture 
                                                                                !! availability (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: harvest_pool            !! The wood and biomass that have been
                                                                                !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_type            !! Type of management that resulted
                                                                                !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_cut             !! Type of cutting that was used for the harvest
                                                                                !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_area            !! Harvested area (m^{2})
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: mai                     !! The mean annual increment
                                                                                !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: pai                     !! The period annual increment
                                                                                !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: previous_wood_volume    !! The volume of the tree trunks
                                                                                !! in a stand for the previous year.
                                                                                !! @tex $(m**3 / m**2 )$ @endtex
    INTEGER(i_std), DIMENSION(:,:),INTENT(inout)     :: mai_count               !! The number of times we've
                                                                                !! calculated the volume increment
                                                                                !! for a stand
    REAL(r_std), DIMENSION(:,:,:),INTENT(inout)      :: bm_sync                 !! The difference betweeen the
                                                                                !! biomass in the circ_classes and
                                                                                !! the total biomass 
                                                                                !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:),INTENT(inout)        :: coppice_dens            !! The density of a coppice at the first
                                                                                !! cutting @tex $( 1 / m**2 )$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: orphan_flux             !! Storage for fluxes of PFTs that no longer
                                                                                !! exist. Following the total destruction of 
                                                                                !! a PFT in LCC (veget_max_new = 0), a flux 
                                                                                !! from before LCC needs a storage
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: wstress_season          !! Water stress factor, based on hum_rel_daily
                                                                                !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: wstress_month           !! Water stress factor, based on hum_rel_daily
                                                                                !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm,nbpools), &
                                       INTENT(inout) :: VectorB                 !! Vector containing the litter increase per
                                                                                !! sechiba time step
                                                                                !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), &
                                       INTENT(inout) :: MatrixV                 !!
    REAL(r_std), DIMENSION(npts,nvm,nbpools), &
                                       INTENT(inout) :: VectorU                 !!
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: c_buried                !! Carbon buried during a land cover change to
                                                                                !! a nobio land type (urban/ice).
                                                                                !!  @tex (gC dt^{-1})$ @endtex

    !! 0.4 Local variables

    INTEGER                                          :: i,j,k,l,istep           !! Indices
    INTEGER                                          :: idia,icut,ifm           !! Indices
    INTEGER                                          :: ipar,iele,imbc          !! Indices
    INTEGER                                          :: ipts,ivm                !! Indices
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements):: bm_alloc                !! Biomass increase, i.e. NPP per plant part
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: tot_bm_to_litter        !! Total conversion of biomass to litter 
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: tot_live_biomass        !! Total living biomass  
                                                                                !! @tex $(gC m{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: tot_turnover            !! Total turnover rate  
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                 :: tot_litter_soil_carb    !! Total soil and litter carbon  
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                 :: tot_litter_carb         !! Total litter carbon 
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                 :: tot_soil_carb           !! Total soil carbon  
                                                                                !! @tex $(gC m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts)                     :: carb_mass_variation     !! Carbon Mass variation  
                                                                                !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                 :: cn_ind                  !! Crown area of individuals 
                                                                                !! @tex $(m^{2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times) &
                                                     :: circ_class_kill         !! Number of trees within a circ class that needs
                                                                                !! to be killed @tex $(ind m^{-2})$ @endtex
                                                                                !! IMPORTANT: See the note in constantes.f90
                                                                                !! regarding the indicies for this variable.
    REAL(r_std), DIMENSION(npts,nvm)                 :: use_reserve             !! Mass taken from carbohydrate reserve 
                                                                                !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                 :: woodmass_ind            !! Woodmass of individuals (gC) 
    REAL(r_std), DIMENSION(npts,nvm,nparts)          :: f_alloc                 !! Fraction that goes into plant part 
                                                                                !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts)                     :: avail_tree              !! Space availability for trees 
                                                                                !! (0 to 1, unitless) 
    REAL(r_std), DIMENSION(npts)                     :: avail_grass             !! Space availability for grasses 
                                                                                !! (0 to 1, unitless) 
    REAL(r_std),DIMENSION(npts,nvm)                  :: veget_max_old           !! "Maximal" coverage fraction of a PFT  
                                                                                !! (LAI-> infinity) on ground (unitless) 
    REAL(r_std), DIMENSION(npts)                     :: vartmp                  !! Temporary 1-D variable used to add history
    REAL(r_std), DIMENSION(npts,nvm)                 :: var_temp                !! Temporary 2-D variable used to add history
    REAL(r_std), DIMENSION(npts,nvm)                 :: histvar                 !! History variables
    REAL(r_std), DIMENSION(npts,nvm)                 :: gammas                  !! Slope for individual tree growth (m)
    REAL(r_std), DIMENSION(npts,nvm)                 :: sigma                   !! Threshold for indivudal tree growth (m) 
    REAL(r_std), DIMENSION(npts,nvm)                 :: lrake_frac              !! TEMP
    REAL(r_std), DIMENSION(npts,nvm)                 :: lai                     !! Leaf area index @tex $(m^2 m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                 :: qm_height               !! Quadratic mean height of vegetation (m)
    REAL(r_std), DIMENSION(npts,nvm)                 :: qm_dia                  !! Quadratic mean diameter of vegetation (m)
    REAL(r_std), DIMENSION(npts,nvm)                 :: rdi                     !! relative density index (ntrees/Nmax)
    REAL(r_std), DIMENSION(npts,nvm)                 :: rdi_target_upper        !! relative density index (ntrees/Nmax)
    REAL(r_std), DIMENSION(npts,nvm)                 :: rdi_target_lower        !! relative density index (ntrees/Nmax)
    REAL(r_std), DIMENSION(npts,nvm)                 :: woody_biomass_below     !! total woody biomass below ground 
                                                                                !! (gC/(m**2)) 
    REAL(r_std), DIMENSION(npts,nvm)                 :: woody_biomass_above     !! total woody biomass above ground
                                                                                !! (gC/(m**2)) 
    REAL(r_std), DIMENSION(npts,nvm)                 :: wood_loss_ab            !! Aboveground wood biomass loss to mortality 
                                                                                !! and export (gC/(m**2)) 
    REAL(r_std), DIMENSION(npts,nvm)                 :: wood_loss_be            !! Belowground wood biomass loss to mortality
                                                                                !! and export (gC/(m**2)) 
    REAL(r_std), DIMENSION(npts,nvm)                 :: var_real                !! Temporary variable for histwrite of 
                                                                                !! integers
    REAL(r_std), DIMENSION(npts,nvm)                 :: shoot_ratio             !! Root:shoot ratio
    REAL(r_std), DIMENSION(npts)                     :: prod_s_total            !! Total products remaining in the short-lived  
                                                                                !! pool after the annual decomposition 
                                                                                !! @tex $(gC)$ @endtex 
    REAL(r_std), DIMENSION(npts)                     :: prod_m_total            !! Total products remaining in the medium-lived
                                                                                !! pool after the annual decomposition 
                                                                                !! @tex $(gC)$ @endtex 
    REAL(r_std), DIMENSION(npts)                     :: prod_l_total            !! Total products remaining in the long-lived
                                                                                !! pool after the annual release 
                                                                                !! @tex $(gC)$ @endtex 
    REAL(r_std), DIMENSION(npts)                     :: cflux_prod_total        !! Total flux from decomposition of the short,
                                                                                !! medium and long lived product pools 
                                                                                !! @tex $(gC year^{-1})$ @endtex
    CHARACTER(30)                                    :: var_name
    CHARACTER(LEN=80)                                :: cut_name                !! To store variables names for I/O
    CHARACTER(LEN=80)                                :: fm_name                 !! To store variables names for I/O
    LOGICAL                                          :: lsync                   !! returns false if the biomass
                                                                                !! is not synced... do we need this?  
                                                                                !! We could equal just handle it in the 
                                                                                !! check_biomass_sync
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements):: check_intern           !! Contains the components of the internal
                                                                                !! mass balance chech for this routine
                                                                                !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: closure_intern          !! Check closure of internal mass balance
                                                                                !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_start              !! Start and end pool of this routine 
                                                                                !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_end                !! Start and end pool of this routine 
                                                                                !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts)                     :: pixel_area              !! surface area of the pixel @tex ($m^{2}$)    
    REAL(r_std), DIMENSION(npts,nvm,nlevels_tot)     :: z_level_photo           !! The height of the levels that we will
                                                                                !! use to calculate the effective LAI for
                                                                                !! the albedo routines and photosynthesis.
                                                                                !! @tex $(m)$ @endtex   
    REAL(r_std), DIMENSION(npts,nvm,nlevels)         :: lai_per_level_temp      !! This is the LAI per vertical level
                                                                                !! @tex $(m^{2} m^{-2})$
    REAL(r_std), DIMENSION(npts)                     :: laipix                  !! LAI at the pixel level for optimization
    REAL(r_std), DIMENSION(npts)                     :: gpppix                  !! GPP at the pixel level for optimisation
    LOGICAL, DIMENSION(npts,nvm)                     :: allow_initpheno         !! Are we allowed to declare the 
                                                                                !! beginning of the growing season?
                                                                                !!  and the end of senescence (true/false)
    REAL(r_std), DIMENSION(npts)                     :: litter_demand           !! The amount of litter which will
                                                                                !! be moved from forest to crop pools
                                                                                !! at the end of the year.
                                                                                !! @tex $( gC year^{-1} )$ @endtex
    
    REAL(r_std), DIMENSION(npts)                     :: mb_err_lcc              !! The mass balance error caused by LCC
                                                                                !! @tex $(gC m^{-2})$ @endtex
!_ ================================================================================================================================

    IF (bavard.GE.2) WRITE(numout,*) 'Entering stomate_lpj',firstcall

    !---TEMP---
    CALL check_biomass_sync("stomate_lpj.f90: Start of LPJ", npts, biomass, &
         circ_class_biomass, circ_class_n, ind, &
         lsync, bm_sync)

    !! This is a whole new approach.  Basically, the functional allocation will
    !! not work with the DGVM.  So instead of hvaing a lot of IF loops all over
    !! this routine checking for that, we are just doing one IF loop.  If you
    !! want the functional allocation, you cannot use the DGVM unless you're
    !! willing to make some major changes.

    IF(control%ok_functional_allocation)THEN

 !! 1. Initializations

       !! 1.1 Initialize variables to zero
       co2_to_bm(:,:) = zero
       co2_fire(:,:) = zero
       npp_daily(:,:) = zero
       resp_maint(:,:) = zero
       resp_growth(:,:) = zero
       bm_to_litter(:,:,:,:) = zero
       cn_ind(:,:) = zero
       woodmass_ind(:,:) = zero
       use_reserve(:,:) = zero
       turnover_daily(:,:,:,:) = zero
       circ_class_kill(:,:,:,:,:) = zero
       bm_alloc(:,:,:,:) = zero
       rdi(:,:) = zero
       rdi_target_upper(:,:) = zero
       rdi_target_lower(:,:) = zero

       !! 1.3  Initialize variables to veget_max
       veget_max_old(:,:) = veget_max(:,:)

       !! 1.4 Initialize check for mass balance closure
       !  The mass balance is calculated at the end of this routine
       !  in section 14. Initial biomass and harvest pool all other
       !  relevant pools were just set to zero.
       pool_start(:,:,:) = zero
       DO ipar = 1,nparts
          DO iele = 1,nelements
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  (biomass(:,:,ipar,iele) * veget_max(:,:))
          ENDDO
       ENDDO
  
       !  Surface area (m2) of the pixel
       pixel_area(:) = resolution(:,1) * resolution(:,2)

       ! The biomass harvest pool shouldn't be multiplied by veget_max
       ! its units are already in gC pixel-1. The total amount for the 
       ! pixel was stored in harvest_pool. Account for the harvest and
       ! wood product pools. There are no longer PFTs 
       pool_start(:,1,icarbon) = pool_start(:,1,icarbon) + &
            ( SUM(SUM(harvest_pool(:,:,:,icarbon),3),2) + &
            SUM(prod_l(:,:),2) + SUM(prod_m(:,:),2) + &
            SUM(prod_s(:,:),2) ) / pixel_area(:)

       !! 1.3 Correct management status if waiting for clearcut (management only
       !! changes after a clearcut, except if it's to become unmanaged)
       DO j = 2,nvm
          
          DO i = 1,npts
             
             IF ( (forest_managed(i,j) /= forest_managed_lastyear(i,j)) .AND. &
                  (forest_managed_lastyear(i,j) > 0) .AND. &
                  & (forest_managed(i,j) > 0) .AND.(age_stand(i,j) > 1) ) THEN
                forest_managed(i,j) = forest_managed_lastyear(i,j)
                
             ENDIF
             
          ENDDO
          
       ENDDO
       
       IF (EndOfYear) forest_managed_lastyear(:,:) = forest_managed(:,:)

       !! 1.4 Update stand age
       ! Here we increment the age of the stand, if it's the end of the year.
       ! Notice that we need to do this before the forestry routines, since the 
       ! forest has had a chance to grow since the last time
       ! we called the routine.  Doing it this way may cause a small problem since
       ! forestry is only done every year.  For example, let us say that a forest
       ! die-off occurs due to natural reasons at the end of December.  The age of
       ! that stand will be set to zero.  However, when the end of year happens
       ! the age will be incremented by one here, even though the forest is only a 
       ! a couple weeks old.  There doesn't seem to be a better
       ! way to do this, though, so a forest age of x should be taken to mean that
       ! the forest is between x-1 and x years old.  I see this only being an issue
       ! in short rotation coppices, but those forests should never have
       ! die off because they are harvested every couple years.
       IF (EndOfYear) THEN
          DO ipts=1,npts
             DO ivm=1,nvm
                IF(veget_max(ipts,ivm) .GT. min_stomate)THEN
                   age_stand(ipts,ivm) = age_stand(ipts,ivm) + 1

                   ! last_cut is similar to stand age, but it measures the time since
                   ! any human intervention, either thinning or clearcutting.
                   last_cut(ipts,ivm) = last_cut(ipts,ivm) + 1

                ELSE
                   age_stand(ipts,ivm) = 0
                   last_cut(ipts,ivm) = 0
                ENDIF
             ENDDO
          ENDDO
       ENDIF
       


 !! 2. Prescribe vegetation characteristics

       !  At the first call this routine takes atmospheric CO2 and converts it
       !  into carbon pools to basically avoid modelling seed germination.
       ! At the first call atmospheric CO2 is converted into biomass pools but
       ! the closed canopy assumption is relaxed by the parameter ::canopy_cover(j).
       ! Given that the functional allocation scheme is constrainted by
       ! allometric relationships, diameter (::dia), and stand density (::ind)
       ! are prognostic variables in this scheme. The relationship between
       ! diameter and stand density is controled by the self-thinning rule.
       ! Height is a diagnostic variable and can be derived from diameter through
       ! allometric relationships
       CALL prescribe_prognostic (npts, veget_max, veget, dt_days, PFTpresent, &
            everywhere, when_growthinit, biomass, leaf_frac, & 
            ind, circ_class_n, circ_class_biomass, co2_to_bm, &
            forest_managed, KF, &            
            senescence, age, npp_longterm, lm_lastyearmax, &
            tau_eff_leaf, tau_eff_sap, tau_eff_root, k_latosa_adapt)

       !---TEMP---
       CALL check_biomass_sync("stomate_lpj.f90: After prescribe", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)

!!$       ! Call this even when DGVM is not activated so that "adapted" and "regenerate"
!!$       ! are kept up to date for the moment when the DGVM is activated.
!!$       CALL constraints (npts, dt_days, t2m_month, t2m_min_daily, &
!!$            when_growthinit, adapted, regenerate)


 !! 3. Phenology

       ! Forests and grasses or the so called natural vegetation in ORCHIDEE
       ! undergo a real phenology based on climatology. This means that the
       ! plants are planted the first day of the year and that they live from
       ! their reserves until the day of bud burst. Crops are planted 
       ! (and harvested) every year. We plant them the day of bud burst. The 
       ! biggest difference is that crops do not need reserves to live
       ! between January 1st and the day of bud burst.
       CALL phenology_prognostic (npts, dt_days, PFTpresent, veget_max, &
            tlong_ref, t2m_month, t2m_week, gpp_daily, &
            maxmoiavail_lastyear, minmoiavail_lastyear, moiavail_month, moiavail_week, &
            gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
            senescence, time_hum_min, biomass, leaf_frac, &
            leaf_age, when_growthinit, co2_to_bm, circ_class_n, &
            circ_class_biomass, ind, KF, allow_initpheno, &
            tau_eff_leaf, tau_eff_sap, tau_eff_root, age, &
            everywhere, npp_longterm, lm_lastyearmax, k_latosa_adapt)

       !---TEMP---
       CALL check_biomass_sync("stomate_lpj.f90: After phenology", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)


 !! 4. Respiration, npp, allocation and age class distribution

      !! 4.1 Allometry based allocation (based on Sitch et al 2003, 
      !  Zaehle et al 2010 and Deleuze 2004)
       CALL growth_fun_all (npts, dt_days, veget_max, veget, PFTpresent, &
            senescence, when_growthinit, t2m_week, &
            gpp_daily, gpp_week, resp_maint_part, resp_maint, &
            resp_growth, npp_daily, biomass, age, &
            leaf_age, leaf_frac, use_reserve, t_photo_stress, &
            lab_fac, ind, rue_longterm, circ_class_n, &
            circ_class_biomass, KF, bm_sync, sigma, &
            gammas, tau_eff_leaf, tau_eff_sap, tau_eff_root, &
            k_latosa_adapt)


       !---TEMP---
       CALL check_biomass_sync("stomate_lpj.f90: After growth", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)

       !! 4.2 Calculate mass balance
       !  All C-fluxes except co2_to_bm are calculated before the code deals with
       !  LCC. So in general it doesn't matter whether we use the fluxes from before
       !  or the fluxes from after LCC as the should be exactely the same except for
       !  the fact that they could have been moved from one PFT to another. There
       !  are two important exceptions to this general rule and these exceptions are
       !  the cause of the need of the variable ::orphan_flux: (1) PFTs that entirely
       !  disappear and PFT's that are newly established (and will therefore take up
       !  co2 from the atmosphere. The first exception results in orphan fluxes which
       !  should be accounted for. The second results in the risk of double counting.
       !  When land cover change occurs, PFTs may be entirely removed. When a PFT
       !  is changed into another PFT, the fluxes i.e. GPP are averaged. However,
       !  when a PFT is removed and becomes PFT1 these fluxes will be lost. We 
       !  could store just these fluxes and apply a correction when needed or we 
       !  can simply store all fluxes for the old veget_max values and use where
       !  needed i.e. in the mass balance (note that additional bookkeeping tricks are
       !  required for co2_to_bm). Also, these fluxes should be shown in 
       !  the histwrite files but not in the restart files.
       orphan_flux(:,:,ivegold,icarbon) = veget_max(:,:)
       orphan_flux(:,:,igpp,icarbon) = gpp_daily(:,:)
       orphan_flux(:,:,ico2bm,icarbon) = co2_to_bm(:,:)
       orphan_flux(:,:,irmain,icarbon) = resp_maint(:,:)
       orphan_flux(:,:,irgrow,icarbon) = resp_growth(:,:)
       orphan_flux(:,:,inpp,icarbon) = npp_daily(:,:)
       orphan_flux(:,:,irhet,icarbon) = resp_hetero(:,:)

 !! 5. Land cover change and land management

       ! We only want to do management at the end of the year. Whether LCC takes place 
       ! before land management appears to be a matter of taste but given that LCC 
       ! results in the (partial) destruction of the PFT it appears somehow logic to 
       ! deal with LCC before FM. After all, it seems unlikely that a land owner will 
       ! first carefully thin a forest and then cut it to turn it into a grassland or 
       ! cropland. 
       !TEMP++++++++++++++++
       lrake_frac(:,:)=zero
       !+++++++++++++++


       IF ( EndOfYear) THEN

          !! 5.1 Age class distribution
          !  Following allocation, the trees have grown and it needs to be
          !  checked whether they are still in the correct age class. If not,
          !  redistribute the age classes in line with a prescribe diameter
          !  threshold for each age class
          CALL age_class_distr(npts, circ_class_n, biomass, circ_class_biomass, &
               veget_max, ind, circ_class_dist, wstress_season, &
               lm_lastyearmax, age, leaf_frac, co2_to_bm, &
               everywhere, litter, carbon, lignin_struc, &
               lignin_wood, bm_to_litter,PFTpresent, when_growthinit,&
               forest_managed, KF, senescence, &
               npp_longterm, gpp_daily, leaf_age, lab_fac, &
               gdd_from_growthinit, gdd_midwinter, time_hum_min, gdd_m5_dormance, &
               ncd_dormance, moiavail_month, moiavail_week, ngd_minus5, &
               gpp_week, resp_maint, resp_growth, npp_daily, &
               use_reserve, t_photo_stress, rue_longterm, mai, pai, &
               mai_count, previous_wood_volume, moiavail_growingseason,&
               MatrixA, MatrixV, VectorB, VectorU, age_stand, last_cut, &
               k_latosa_adapt)

          !---TEMP---
          CALL check_biomass_sync("stomate_lpj.f90: After redistributing age classes", &
               npts, biomass, circ_class_biomass, circ_class_n, ind, &
               lsync, bm_sync)

          !! 5.2 Land cover change
          IF (control%ok_land_cover_change) THEN

             !+++CHECK+++
             ! NOTE: when this code was written, ::veget_cov_max_new was passed into 
             ! stomate_lpj. This has now changed and only ::veget_max is passed so 
             ! the checks on the vegetation cover in the code need to be checked.
             CALL land_cover_change_main (npts, dt_days, veget_max, veget, harvest_pool, &
                  harvest_type, harvest_cut, harvest_area, &
                  litter, carbon, lignin_struc, lignin_wood, &
                  biomass, PFTpresent, everywhere, when_growthinit, & 
                  leaf_frac, ind, circ_class_n, circ_class_biomass, &
                  co2_to_bm, forest_managed, &
                  KF, wstress_season, wstress_month, senescence, npp_longterm, &
                  age, lm_lastyearmax, harvest_pool_bound, resolution, &
                  bm_to_litter, leaf_age, circ_class_dist, orphan_flux, &
                  bm_sync, vir_wstress_fac, tau_eff_leaf, tau_eff_sap, &
                  tau_eff_root, veget_max_new, age_stand, last_cut, &
                  c_buried, mb_err_lcc, k_latosa_adapt)
             !++++++++++

             ! TEMP
             CALL histwrite_p (hist_id_stomate, 'MB_ERR_LCC', itime, &
                  mb_err_lcc(:), npts, hori_index)



             !---TEMP---
             CALL check_biomass_sync("stomate_lpj.f90: After land cover change", &
                  npts, biomass, circ_class_biomass, circ_class_n, ind, &
                  lsync, bm_sync)

          ENDIF

          !! 5.3 Forest management
          CALL forestry (npts, biomass, age_stand, last_cut, &
               circ_class_n, circ_class_kill, forest_managed, rdi,&
               circ_class_biomass, mai, pai, previous_wood_volume, &
               mai_count, coppice_dens, veget_max, rdi_target_upper,&
               rdi_target_lower)

          !---TEMP---
          CALL check_biomass_sync("stomate_lpj.f90: After forest management", &
               npts, biomass, circ_class_biomass, circ_class_n, ind, &
               lsync, bm_sync)

          !! 5.3 Crop harvest
          !  According to the logic of this module crop harvest should 
          !  take place here but it depends on :: turnover_daily and that
          !  variable is not calculated until the call to turnover. Also
          !  the implemented approach harvests biomass daily rather once
          !  per year (as it should be).

          !! 5.4 Grazing
          !  This is an excellent place to account for grazing.

          !! 5.5 Litter raking
          !  In order to feed animals through the winter, as well as absorb
          !  their manure to spread on the fields during the spring, farmers
          !  in historical Europe would collect litter from forests and place
          !  it in the stables during winter.  We will simulate this by
          !  looking at the litter pools at the end of every year and moving
          !  around a certain amount of litter from the forest to the crop PFTs.
          !  We do this after the forest management under the assumption that
          !  branches left on site after thinning operations would, historically,
          !  have been collected for this purpose.  Of course, litter raking fell
          !  out of fashion in the mid-19th century, around the same time that
          !  scientific silvaculture became popular, so perhaps it doesn't matter.
          !  If you are not doing historical runs, litter raking almost certainly
          !  does not apply to you.
          IF(use_litter_raking)THEN
             CALL litter_raking(npts, veget_max, resolution, litter_demand, litter, lrake_frac)
          ENDIF

       ENDIF ! EndOfYear


 !! 6. Kill PFT's

       ! Kill slow growing PFTs in DGVM or STOMATE with dynamic or
       ! constant mortality. If the leaf, root or reserve pool are zero 
       ! kill the PFT, move the biomass to the litter pool and reinitialize 
       ! some PFT variables
 
       CALL kill_prognostic (npts, 'growth    ', lm_lastyearmax,  ind, &
            PFTpresent, biomass, npp_longterm, circ_class_biomass, &
            circ_class_n, circ_class_kill, rue_longterm, turnover_longterm, &
            dt_days, veget_max, st_dist)


       !---TEMP---
       CALL check_biomass_sync("stomate_lpj.f90: After kill", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)


 !! 7. Fire
 
      IF (.NOT. control%ok_constant_mortality) THEN

          ! +++CHECK+++
          ! The current call to fire does not synchronize ind and circ_class_n
          ! and biomass and circ_class_biomass yet. This should be done first.
          ! +++++++++++

       ELSE

          ! The PFTs do not burn when constant mortality is used 
          ! outside the DGVM. Write the values of the flags to make 
          ! sure this condition is satisfied
          IF (firstcall) THEN

!!$             WRITE(numout,*) 'PFTs are not killed and the vegatation updated because '
!!$             WRITE(numout,*) 'constant mortality is used outside the DGVM'
!!$             WRITE(numout,*) 'control%ok_functional_allocation, ', control%ok_functional_allocation
!!$             WRITE(numout,*) 'control%ok_dgvm, ', control%ok_dgvm
!!$             WRITE(numout,*) 'control%ok_constant_mortality, ', control%ok_constant_mortality
!!$             WRITE(numout,*) 'disable_fire, ', disable_fire

          ENDIF

       ENDIF  ! constant mortality


 !! 8. Tree mortality

       ! Here is where the trees actually die and biomass is moved around.  
       ! First, we kill all the trees that died because of human intervention.
       CALL gap_prog_sapiens (npts, biomass, ind, &
            bm_to_litter, circ_class_biomass, circ_class_kill, &
            circ_class_n, harvest_pool_bound, harvest_pool, harvest_type, &
            harvest_cut, harvest_area, resolution, &
            veget_max, age_stand, last_cut, mai_count, circ_class_dist, &
            coppice_dens)

       !---TEMP---
       CALL check_biomass_sync("stomate_lpj.f90: After gap_prog_sapiens", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)

       ! Now we kill all the plants which died due to natural causes.  This comes
       ! after the plant death due to human causes, since in theory a forest
       ! that is thinned will not suffer from as high of natural mortality.
       CALL gap_prognostic (npts, biomass, ind, bm_to_litter, &
            circ_class_biomass, circ_class_kill, circ_class_n, &
            veget_max)

       !---TEMP---
       CALL check_biomass_sync("stomate_lpj.f90: After gap_prognostic", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)


       ! Now we need to do some checking.  If there is no biomass on the site,
       ! some variables have to be reset.  If some of the circumference classes
       ! are empty, we need to redistribute the biomass among them.  Notice this
       ! must be done after both the natural and human killing is done, because
       ! circ_class_kill for the natural killing is calculated based on 
       ! circ_class_biomass, and if we redistribute we change circ_class_biomass.
       CALL gap_clean (npts, biomass, circ_class_biomass, &
            circ_class_n, age, everywhere, leaf_age,&
            leaf_frac, senescence, when_growthinit, circ_class_dist,&
            age_stand, PFTpresent, last_cut, mai_count, &
            veget_max, npp_longterm, KF, co2_to_bm, ind, &
            wstress_season, lm_lastyearmax, lignin_struc, lignin_wood, &
            carbon, litter,  vir_wstress_fac, veget, dt_days, &
            forest_managed, bm_to_litter, lab_fac, gdd_from_growthinit,&
            gdd_midwinter, time_hum_min, gdd_m5_dormance, ncd_dormance,&
            moiavail_month, moiavail_week, gpp_week, gpp_daily, resp_maint,&
            resp_growth, npp_daily, use_reserve, mai, pai, ngd_minus5,&
            t_photo_stress, rue_longterm, previous_wood_volume,&
            tau_eff_leaf, tau_eff_sap, tau_eff_root, moiavail_growingseason,&
            orphan_flux, k_latosa_adapt)

       !---TEMP---
       CALL check_biomass_sync("stomate_lpj.f90: After gap_clean", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)


 !! 9. Calculate vcmax, vjmax and photosynthesis temperatures

       CALL vmax_prognostic (npts, dt_days, leaf_age, leaf_frac, &
            age_stand, vcmax, tau_eff_leaf)

       CALL assim_temp (npts, tlong_ref, t2m_month, t_photo_min, &
            t_photo_opt, t_photo_max)


 !! 10. Leaf senescence, turnover and crop harvest

       ! For trees and grasses there is a real turnover and senescence can
       ! last days to weeks. For crops the end of the growing season is 
       ! followed by a harvest and part of the crop biomass is added to the
       ! harvest pool.
       CALL turnover_prognostic (npts, dt_days, PFTpresent, herbivores, &
            maxmoiavail_lastyear, minmoiavail_lastyear, moiavail_week,  moiavail_month, &
            tlong_ref, t2m_longterm, t2m_month, t2m_week, veget_max, &
            gdd_from_growthinit, leaf_age, leaf_frac, age, &
            biomass, turnover_daily, senescence, turnover_time, &
            circ_class_biomass, circ_class_n, ind, allow_initpheno, &
            when_growthinit,tau_eff_leaf, tau_eff_sap, tau_eff_root, &
            harvest_pool, harvest_type, harvest_cut, harvest_area, &
            resolution, wstress_month)
      

       !---TEMP---
       CALL check_biomass_sync("stomate_lpj.f90: After turnover", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)

       
 !! 11. Product use
 
       !  +++CHECK+++
       !  Write the harvest pools to the history files here because they will
       !  be reset when the subroutine basic_product_use is called. Note that
       !  due to some peculiarities in the coding the units of the harvest 
       !  pool differ for grasses, crops and forests. For forest, the 
       !  harvest_pool is filled and emptied the last day of the year. Hence
       !  all daily values are 0, except for the last day. When writing,
       !  for example, monthly output files the mean monthly value will be 
       !  written. To obtain the absolute harvest, the value in the history
       !  file should be multiplied by 30 or 31 depending on whether a 360 or 
       !  365 day calendre is used. For grasslands the harvest_pool is filled
       !  daily with the turnover. So the mean value written to the history file
       !  is the correct value. Crops are typically harvested before December
       !  31st but the harvest_pool is only emptied on December 31st (just as
       !  for grasses and forests). So between the harvest date and December
       !  31st the same biomass remains in the harvest pools. If the harvest 
       !  takes place before December 1st the same average value is written for
       !  every day of December and therefore the last month contains the correct
       !  value for harvest.
       !  All of this would become straightforward if we would only write harvest_pool
       !  the last day of the year. We tried using the operator once in histwrite but
       !  were unhappy with the outcome. The IPSL team no longer maintains histwrite
       !  as it should get replaced by the ioxserver.
       var_temp = SUM(harvest_pool(:,:,:,icarbon),3)
       CALL histwrite_p (hist_id_stomate, 'HARVEST_POOL_C', itime, &
            var_temp, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'HARVEST_TYPE', itime, &
            harvest_type(:,:), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'HARVEST_CUT', itime, &
            harvest_cut(:,:), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'HARVEST_AREA', itime, &
            harvest_area(:,:), npts*nvm, horipft_index)
       !+++++++++++

       !  Agricultural harvest now goes into the short-lived product pool.
       !  Hence, product use can only be estimated after crop_harvest which
       !  in turn depends on the calculation of turnover. In previous version
       !  product use was an integral part of LCC. Maintaining this approach 
       !  would result in inconsistencies with harvest from forestry and 
       !  agriculture. Note that there is still no real harvest from grasslands. 
       !  Grasslands are thus wrongly considered natural PFTs without any human 
       !  use. To partly overcome this issue the turnover in grasslands is moved 
       !  into the harvest pool but this approach should be considered as a
       !  patch rather than a permanent solution.
       IF(EndofYear)THEN
 
          IF (control%ok_dofoco) THEN

             ! Wood harvest with small dimensions is being used as fire wood
             ! the remaining wood goes into the short, medium and long product
             ! pools. This implies that when the dimensions of the harvest 
             ! change, the product pools will change as well
             CALL dim_product_use(npts, dt_days, harvest_pool_bound, harvest_pool, &
                  harvest_type, harvest_cut, harvest_area, &
                  cflux_prod_s, cflux_prod_m, cflux_prod_l, prod_s, &
                  prod_m, prod_l, prod_s_total, prod_m_total, &
                  prod_l_total, cflux_prod_total, flux_s, flux_m, &
                  flux_l, resolution) 

          ELSE

             ! The scheme does not account for the harvest dimensions. All
             ! wood is distributed across the short, medium and long product
             ! pools. This basically means that wood use in 1600, 1700, 1800,
             ! 1900 and 2000 was identical.
             CALL basic_product_use(npts, dt_days, harvest_pool_bound, harvest_pool, &
                  harvest_type, harvest_cut, harvest_area, &
                  cflux_prod_s, cflux_prod_m, cflux_prod_l, prod_s, &
                  prod_m, prod_l, prod_s_total, prod_m_total, &
                  prod_l_total, cflux_prod_total, flux_s, flux_m, &
                  flux_l, resolution)

          ENDIF ! control%ok_dofoco

        ELSE
    
          ! These variables are all intent(out) so they should be initialized 
          ! if it's not the end of the year
          CALL basic_product_init(cflux_prod_s, cflux_prod_m, cflux_prod_l, &
               prod_s_total, prod_m_total, prod_l_total, flux_s, &
               flux_m, flux_l, cflux_prod_total)

       ENDIF

       !---TEMP---
       CALL check_biomass_sync("stomate_lpj.f90: After basic_product_use", &
            npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)


 !! 12. Establishment of saplings

       IF (.NOT. control%ok_constant_mortality) THEN

          ! Establish new plants based on the functional allocation
          ! +++CHECK+++
          ! How does this intefer with self-thinning it seems that this code tries 
          ! to maintain a +/- high biomass by adding individuals when the fpc (foliage
          ! projected cover decreases). This is exactely what we want to avoid as we
          ! want to introduce gaps in the static approach. We could try to adjust the 
          ! proposed aproach for unmanaged systems but I doubt whether the rule of 
          ! Deleuze and Dhote could deal with such an extreme range of diameter sizes
          ! Most unmanged forest are under a disturbance regime and may be aproximated 
          ! by an 'even-age' evolution.

          ! NOTE:  We have decided not to create this routine.  The reasoning is that
          ! we really want gaps in the system and don't want to continually establish.
          ! All of this should be taken care of in prescribe_prognostic at the end
          ! of the year.  If someone wants to run with DGVM and the new allocation,
          ! this will require some effort here.

!!$       CALL establish_prognostic (npts, dt_days, PFTpresent, regenerate, &
!!$            neighbours, resolution, need_adjacent, herbivores, &
!!$            precip_lastyear, gdd0_lastyear, lm_lastyearmax, avail_tree, &
!!$            avail_grass, npp_longterm, leaf_age, leaf_frac, &
!!$            ind, biomass, age, everywhere, &
!!$            co2_to_bm, veget_max, circ_class_biomass, circ_class_n)
          ! ++++++++++++

       ELSE

          ! Plant establishment does not occur when constant mortality 
          ! is used outside the DGVM. Write the values of the flags to 
          ! make sure this condition is satisfied
          IF (firstcall) THEN

             WRITE(numout,*) 'Plant establishment does not occur because '
             WRITE(numout,*) 'constant mortality is used outside the DGVM'
             WRITE(numout,*) 'control%ok_functional_allocation, ', &
                  control%ok_functional_allocation
             WRITE(numout,*) 'control%ok_dgvm, ', control%ok_dgvm
             WRITE(numout,*) 'control%ok_constant_mortality, ', &
                  control%ok_constant_mortality

          ENDIF

       ENDIF  ! constant mortality 

       !---TEMP---
       CALL check_biomass_sync("stomate_lpj.f90: After establish", npts, biomass, &
            circ_class_biomass, circ_class_n, ind, &
            lsync, bm_sync)


  !! 13. Compute the effective leaf area index. 
  
       ! This is a function of the solar angle, and needed for the albedo
       ! and the photosynthesis.  It was very expensive to compute, so instead
       ! of computing it at every timestep we compute it at a couple points
       ! throughout the day and fit a function to it.  We do this now because
       ! the canopy as it appears now will be used for the next day.

       ! I also need to put in a function here to compute the height of the photosynthesis
       ! levels given the height of the energy levels and the vegetation on the grid 
       ! square.  The hightest levels will be a function of the height of the vegetation
       ! so that we don't waste computational time on empty levels.
       CALL calculate_z_level_photo(npts, &
            circ_class_biomass, &
            circ_class_n, biomass, z_level_photo)
       
       ! Finding the true LAI per level is different from finding the
       ! effective LAI per level, so we'll do that here.  This only
       ! changes once every day so hopefully it is not too expensive.
       ! It will eventually be needed by the energy budget.  It is
       ! also needed by effective_lai for grasses and crops.
       CALL find_lai_per_level(npts, nlevels_tot, z_level_photo, &
            biomass, circ_class_biomass, &
            circ_class_n, lai_per_level)
       
       ! Now we actually find the effective LAI and fit the function 
       ! we'll use later on 
       CALL fitting_laieff(npts, nlevels_tot, z_level_photo, circ_class_biomass, & 
            circ_class_n, veget_max, biomass, lai_per_level, laieff_fit) 
	 	 	 

  !! 14. Check mass balance closure
       
       !! 15.1 Calculate components of the mass balance
       pool_end(:,:,:) = zero 
       DO ipar = 1,nparts
          DO iele = 1,nelements
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  (biomass(:,:,ipar,iele) + bm_to_litter(:,:,ipar,iele) + &
                  turnover_daily(:,:,ipar,iele)) * veget_max(:,:)
          ENDDO
       ENDDO
       
       pool_end(:,1,icarbon) = pool_end(:,1,icarbon) + &
            ( SUM(SUM(harvest_pool(:,:,:,icarbon),3),2) + &
            SUM(prod_l(:,:),2) + SUM(prod_m(:,:),2) + &
            SUM(prod_s(:,:),2) ) / pixel_area(:)

       !! 15.2 Calculate mass balance
       !  iatm2land and iland2atm have already been calculated after growth in §4.2
       ! For the orphan fluxes, ivegnew is just the veget_max as it currently is.
       ! If the veget_max has not changed, the orphan_fluxes for ivegold and ivegnew
       ! are exactly the same.
       !+++ +CHECK ++++
       ! We will probably need two orphan fluxes...one for gap_clean and one for
       ! LCC.  So right now we should not trust stomate_lpj closure.  However,
       ! all of the routines in stomate_lpj should be closed.
       !++++++++++++++
       orphan_flux(:,:,ivegnew,icarbon)=veget_max(:,:)

       ! Use orphan fluxes here as well.  In the case of age classes, if an old
       ! PFT dies, it will regrow as the youngest PFT, requiring the use of an 
       ! orphan flux.  If this doesn't happen, the orphan fluxes are identical
       ! to GPP, co2bm, and veget_max, so the equation collapses to the correct one.
       check_intern(:,:,iatm2land,icarbon) = &
            orphan_flux(:,:,igpp,icarbon) * orphan_flux(:,:,ivegold,icarbon) +  &
            orphan_flux(:,:,ico2bm,icarbon) * orphan_flux(:,:,ivegnew,icarbon)
       check_intern(:,:,iland2atm,icarbon) = -un * (orphan_flux(:,:,irmain,icarbon) + &
                  orphan_flux(:,:,irgrow,icarbon)) * orphan_flux(:,:,ivegold,icarbon)
       check_intern(:,1,iland2atm,icarbon) = -un * ( SUM(flux_s(:,:),2) + SUM(flux_m(:,:),2) + & 
            SUM(flux_l(:,:),2) ) / pixel_area(:)
       check_intern(:,:,ilat2out,icarbon) = zero
       check_intern(:,:,ilat2in,icarbon) = -un * zero
       check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
       closure_intern = zero
       DO imbc = 1,nmbcomp
          closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
       ENDDO

       !! 15.3 Write outcome of the check
       !  Sum over ivm because of LCC
       DO ipts=1,npts
          IF(ABS(SUM(closure_intern(ipts,:,icarbon))) .LE. min_stomate)THEN
             IF (ld_massbal) WRITE(numout,*) 'Mass balance closure in stomate_lpj.f90'
          ELSE
             WRITE(numout,*) 'Error: mass balance is not closed in stomate_lpj.f90'
             WRITE(numout,*) 'Total difference: ',SUM(closure_intern(ipts,:,icarbon))
             DO ivm=1,nvm
                WRITE(numout,*) '   ipts, ivm: ', ipts,ivm
                WRITE(numout,*) '   Difference is, ', closure_intern(ipts,ivm,icarbon)
                WRITE(numout,*) '   pool_end,pool_start: ', pool_end(ipts,ivm,icarbon), pool_start(ipts,ivm,icarbon)
                WRITE(numout,*) '   orphan_fluxes: ', orphan_flux(ipts,ivm,irmain,icarbon),&
                     orphan_flux(ipts,ivm,irgrow,icarbon),orphan_flux(ipts,ivm,ivegold,icarbon)
             !   IF (control%ok_land_cover_change .AND. EndOfYear) THEN
                    WRITE(numout,*) '   orphan_fluxes LCC (total): ',check_intern(ipts,ivm,iatm2land,icarbon)
                    WRITE(numout,*) '   orphan_fluxes LCC: ',&
                        orphan_flux(ipts,ivm,igpp,icarbon), orphan_flux(ipts,ivm,ivegold,icarbon),  &
                        orphan_flux(ipts,ivm,ico2bm,icarbon), orphan_flux(ipts,ivm,ivegnew,icarbon)
             !   ELSE
                    WRITE(numout,*) '   atm2land (total): ',check_intern(ipts,ivm,iatm2land,icarbon)
                    WRITE(numout,*) '   gpp,co2_to_bm,veget_max: ',gpp_daily(ipts,ivm), co2_to_bm(ipts,ivm), &
                        veget_max(ipts,ivm)
             !   ENDIF
            ENDDO
             WRITE(numout,*) '   flux_s: ', flux_s(ipts,:)
             WRITE(numout,*) '   flux_m: ', flux_m(ipts,:)
             WRITE(numout,*) '   flux_l: ', flux_l(ipts,:)
             WRITE(numout,*) '   pixel_area: ', pixel_area(ipts)
             IF(ld_stop)THEN
                CALL ipslerr_p (3,'stomatelpg', 'Mass balance error.','','')
             ENDIF
          ENDIF
       ENDDO

 !! 15. Write history

       !! 16.1 Write Esiting variables ioserver
       CALL xios_orchidee_send_field("RESOLUTION_X",resolution(:,1))
       CALL xios_orchidee_send_field("RESOLUTION_Y",resolution(:,2))
       CALL xios_orchidee_send_field("CONTFRAC_STOMATE",contfrac(:))
       CALL xios_orchidee_send_field("T2M_MONTH",t2m_month)
       CALL xios_orchidee_send_field("T2M_WEEK",t2m_week)
       CALL xios_orchidee_send_field("HET_RESP",resp_hetero(:,:))
       CALL xios_orchidee_send_field("CO2_FIRE",co2_fire)
       CALL xios_orchidee_send_field("CO2_TAKEN",co2_to_bm)
       CALL xios_orchidee_send_field("VEGET_MAX",veget_max)
       CALL xios_orchidee_send_field("NPP_STOMATE",npp_daily)
       CALL xios_orchidee_send_field("GPP",gpp_daily)
       CALL xios_orchidee_send_field("IND",ind)
       CALL xios_orchidee_send_field("CN_IND",cn_ind)
       CALL xios_orchidee_send_field("WOODMASS_IND",woodmass_ind)
       CALL xios_orchidee_send_field("MOISTRESS",moiavail_week)
       CALL xios_orchidee_send_field("LEAF_M",biomass(:,:,ileaf,icarbon))
       CALL xios_orchidee_send_field("SAP_M_AB",biomass(:,:,isapabove,icarbon))
       CALL xios_orchidee_send_field("SAP_M_BE",biomass(:,:,isapbelow,icarbon))
       CALL xios_orchidee_send_field("HEART_M_AB",biomass(:,:,iheartabove,icarbon))
       CALL xios_orchidee_send_field("HEART_M_BE",biomass(:,:,iheartbelow,icarbon))
       CALL xios_orchidee_send_field("ROOT_M",biomass(:,:,iroot,icarbon))
       CALL xios_orchidee_send_field("FRUIT_M",biomass(:,:,ifruit,icarbon))
       CALL xios_orchidee_send_field("RESERVE_M",biomass(:,:,icarbres,icarbon))
       CALL xios_orchidee_send_field("LEAF_TURN",turnover_daily(:,:,ileaf,icarbon))
       CALL xios_orchidee_send_field("MAINT_RESP",resp_maint)
       CALL xios_orchidee_send_field("GROWTH_RESP",resp_growth)
       CALL xios_orchidee_send_field("SAP_AB_TURN",turnover_daily(:,:,isapabove,icarbon))
       CALL xios_orchidee_send_field("ROOT_TURN",turnover_daily(:,:,iroot,icarbon))
       CALL xios_orchidee_send_field("FRUIT_TURN",turnover_daily(:,:,ifruit,icarbon))
       CALL xios_orchidee_send_field("LEAF_BM_LITTER",bm_to_litter(:,:,ileaf,icarbon))
       CALL xios_orchidee_send_field("SAP_AB_BM_LITTER",bm_to_litter(:,:,isapabove,icarbon))
       CALL xios_orchidee_send_field("SAP_BE_BM_LITTER",bm_to_litter(:,:,isapbelow,icarbon))
       CALL xios_orchidee_send_field("HEART_AB_BM_LITTER",bm_to_litter(:,:,iheartabove,icarbon))
       CALL xios_orchidee_send_field("HEART_BE_BM_LITTER",bm_to_litter(:,:,iheartbelow,icarbon))
       CALL xios_orchidee_send_field("ROOT_BM_LITTER",bm_to_litter(:,:,iroot,icarbon))
       CALL xios_orchidee_send_field("FRUIT_BM_LITTER",bm_to_litter(:,:,ifruit,icarbon))
       CALL xios_orchidee_send_field("RESERVE_BM_LITTER",bm_to_litter(:,:,icarbres,icarbon))
       CALL xios_orchidee_send_field("LITTER_STR_AB",litter(:,istructural,:,iabove,icarbon))
       CALL xios_orchidee_send_field("LITTER_MET_AB",litter(:,imetabolic,:,iabove,icarbon))
       CALL xios_orchidee_send_field("LITTER_STR_BE",litter(:,istructural,:,ibelow,icarbon))
       CALL xios_orchidee_send_field("LITTER_MET_BE",litter(:,imetabolic,:,ibelow,icarbon))
       CALL xios_orchidee_send_field("DEADLEAF_COVER",deadleaf_cover)
       CALL xios_orchidee_send_field("CARBON_ACTIVE",carbon(:,iactive,:))
       CALL xios_orchidee_send_field("CARBON_SLOW",carbon(:,islow,:))
       CALL xios_orchidee_send_field("CARBON_PASSIVE",carbon(:,ipassive,:))
       CALL xios_orchidee_send_field("LITTERHUM",litterhum_daily)
       CALL xios_orchidee_send_field("TURNOVER_TIME",turnover_time)
       ! Some of these are commented out because they are the result of
       ! a merge from the trunk, but we don't have those variables anymore.
       !    CALL xios_orchidee_send_field("PROD10",prod10)
       !    CALL xios_orchidee_send_field("FLUX10",flux10)
       !    CALL xios_orchidee_send_field("PROD100",prod100)
       !    CALL xios_orchidee_send_field("FLUX100",flux100)
       !    CALL xios_orchidee_send_field("CONVFLUX",convflux)
       !    CALL xios_orchidee_send_field("CFLUX_PROD10",cflux_prod10)
       !    CALL xios_orchidee_send_field("CFLUX_PROD100",cflux_prod100)
       !    CALL xios_orchidee_send_field("HARVEST_ABOVE",harvest_above)
       CALL xios_orchidee_send_field("VCMAX",vcmax)
       CALL xios_orchidee_send_field("AGE",age)
       !    CALL xios_orchidee_send_field("HEIGHT",height)
       CALL xios_orchidee_send_field("BLACK_CARBON",black_carbon)
       CALL xios_orchidee_send_field("FIREINDEX",fireindex(:,:))
       ! ipcc history
       !     CALL xios_orchidee_send_field("cProduct",(prod10_total + prod100_total)/1e3)
       CALL xios_orchidee_send_field("gpp_ipcc",SUM(gpp_daily*veget_max,dim=2)/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("ra",SUM((resp_maint+resp_growth)*veget_max,dim=2)/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("npp_ipcc",SUM(npp_daily*veget_max,dim=2)/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("rh",SUM(resp_hetero*veget_max,dim=2)/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("fFire",SUM(co2_fire*veget_max,dim=2)/1e3/one_day*contfrac)
       !     CALL xios_orchidee_send_field("fHarvest",harvest_above/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("fLuc",cflux_prod_total/1e3/one_day*contfrac)
       !     CALL xios_orchidee_send_field("nbp",(SUM((gpp_daily-(resp_maint+resp_growth+resp_hetero)-co2_fire) &
       !            &        *veget_max,dim=2)-cflux_prod_total-harvest_above)/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("fLitterSoil",SUM(SUM(soilcarbon_input,dim=2)*veget_max,dim=2)/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("cLeaf",SUM(biomass(:,:,ileaf,icarbon)*veget_max,dim=2)/1e3*contfrac)
       CALL xios_orchidee_send_field("cWood",SUM((biomass(:,:,isapabove,icarbon)+biomass(:,:,iheartabove,icarbon))*&
            veget_max,dim=2)/1e3*contfrac)
       CALL xios_orchidee_send_field("cRoot",SUM(( biomass(:,:,iroot,icarbon) + biomass(:,:,isapbelow,icarbon) + &
            biomass(:,:,iheartbelow,icarbon) )*veget_max,dim=2)/1e3*contfrac)
       CALL xios_orchidee_send_field("cMisc",SUM(( biomass(:,:,icarbres,icarbon) + biomass(:,:,ifruit,icarbon))*&
            veget_max,dim=2)/1e3*contfrac)
       CALL xios_orchidee_send_field("cLitterAbove",SUM((litter(:,istructural,:,iabove,icarbon)+&
            litter(:,imetabolic,:,iabove,icarbon))*veget_max,dim=2)/1e3*contfrac)
       CALL xios_orchidee_send_field("cLitterBelow",SUM((litter(:,istructural,:,ibelow,icarbon)+&
            litter(:,imetabolic,:,ibelow,icarbon))*veget_max,dim=2)/1e3*contfrac)
       CALL xios_orchidee_send_field("cSoilFast",SUM(carbon(:,iactive,:)*veget_max,dim=2)/1e3*contfrac)
       CALL xios_orchidee_send_field("cSoilMedium",SUM(carbon(:,islow,:)*veget_max,dim=2)/1e3*contfrac)
       CALL xios_orchidee_send_field("cSoilSlow",SUM(carbon(:,ipassive,:)*veget_max,dim=2)/1e3*contfrac)
       DO j=1,nvm
          histvar(:,j)=veget_max(:,j)*contfrac(:)*100
       ENDDO
       CALL xios_orchidee_send_field("landCoverFrac",histvar)
       vartmp(:)=zero
       DO j = 2,nvm
          IF (is_deciduous(j)) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
          ENDIF
       ENDDO
       CALL xios_orchidee_send_field("treeFracPrimDec",vartmp)
       vartmp(:)=zero
       DO j = 2,nvm
          IF (is_evergreen(j)) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
          ENDIF
       ENDDO
       CALL xios_orchidee_send_field("treeFracPrimEver",vartmp)
       vartmp(:)=zero
       DO j = 2,nvm
          IF ( .NOT.(is_c4(j)) ) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
          ENDIF
       ENDDO
       CALL xios_orchidee_send_field("c3PftFrac",vartmp)
       vartmp(:)=zero
       DO j = 2,nvm
          IF ( is_c4(j) ) THEN
             vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
          ENDIF
       ENDDO
       CALL xios_orchidee_send_field("c4PftFrac",vartmp)
       CALL xios_orchidee_send_field("rGrowth",SUM(resp_growth*veget_max,dim=2)/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("rMaint",SUM(resp_maint*veget_max,dim=2)/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("nppLeaf",SUM(bm_alloc(:,:,ileaf,icarbon)*veget_max,dim=2)/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("nppWood",SUM(bm_alloc(:,:,isapabove,icarbon)*veget_max,dim=2)/1e3/one_day*contfrac)
       CALL xios_orchidee_send_field("nppRoot",SUM(( bm_alloc(:,:,isapbelow,icarbon) + bm_alloc(:,:,iroot,icarbon) )*&
            veget_max,dim=2)/1e3/one_day*contfrac)     

       !++++++ TEMP +++++++
      CALL histwrite_p (hist_id_stomate, 'LIGNIN_STRUC_AB', itime, &
            lignin_struc(:,:,iabove), npts*nvm, horipft_index)
      CALL histwrite_p (hist_id_stomate, 'LIGNIN_STRUC_BE', itime, &
            lignin_struc(:,:,ibelow), npts*nvm, horipft_index)
      CALL histwrite_p (hist_id_stomate, 'LIGNIN_WOOD_AB', itime, &
            lignin_wood(:,:,iabove), npts*nvm, horipft_index)
      CALL histwrite_p (hist_id_stomate, 'LIGNIN_WOOD_BE', itime, &
            lignin_wood(:,:,ibelow), npts*nvm, horipft_index)
      
       !+++++++++++++++++++

       CALL histwrite_p (hist_id_stomate, 'RESOLUTION_X', itime, &
            resolution(:,1), npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'RESOLUTION_Y', itime, &
            resolution(:,2), npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'CONTFRAC', itime, &
            contfrac(:), npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'LITTER_STR_AB', itime, &
            litter(:,istructural,:,iabove,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'LITTER_MET_AB', itime, &
            litter(:,imetabolic,:,iabove,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'LITTER_STR_BE', itime, &
            litter(:,istructural,:,ibelow,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'LITTER_MET_BE', itime, &
            litter(:,imetabolic,:,ibelow,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'LITTER_WOD_AB', itime, &
            litter(:,iwoody,:,iabove,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'LITTER_WOD_BE', itime, &
            litter(:,iwoody,:,ibelow,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'DEADLEAF_COVER', itime, &
            deadleaf_cover, npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'CARBON_ACTIVE', itime, &
            carbon(:,iactive,:), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'CARBON_SLOW', itime, &
            carbon(:,islow,:), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'CARBON_PASSIVE', itime, &
            carbon(:,ipassive,:), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'T2M_MONTH', itime, &
            t2m_month, npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'T2M_WEEK', itime, &
            t2m_week, npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'HET_RESP', itime, &
            resp_hetero(:,:), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'BLACK_CARBON', itime, &
            black_carbon, npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'FIREINDEX', itime, &
            fireindex(:,:), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'LITTERHUM', itime, &
            litterhum_daily, npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'CO2_FIRE', itime, &
            co2_fire, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'CO2_TAKEN', itime, &
            co2_to_bm, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'RDI', itime, &
             rdi, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'RDI_TARGET_UPPER', itime, &
            rdi_target_upper, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'RDI_TARGET_LOWER', itime, &
         rdi_target_lower, npts*nvm, horipft_index)

       ! Calculate lai so it can be written to the history files
       DO ivm = 2,nvm
          lai(:,ivm) = biomass(:,ivm,ileaf,icarbon) * sla(ivm)
          DO ipts = 1,npts
             qm_dia(ipts,ivm) = wood_to_qmdia(circ_class_biomass(ipts,ivm,:,:,icarbon), &
                  circ_class_n(ipts,ivm,:), ivm)
             qm_height(ipts,ivm) = wood_to_qmheight(circ_class_biomass(ipts,ivm,:,:,icarbon), &
                  circ_class_n(ipts,ivm,:), ivm)
          ENDDO
       ENDDO
       lai(:,1) = zero
       qm_dia(:,1) = zero
       qm_height(:,1) = zero

       CALL xios_orchidee_send_field("LAI",lai)
       CALL xios_orchidee_send_field("lai_ipcc",SUM(lai*veget_max,dim=2)*contfrac)
       CALL histwrite_p (hist_id_stomate, 'LAI', itime, &
            lai, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'VEGET_MAX', itime, &
            veget_max, npts*nvm, horipft_index)
      
       ! Calculate and write lai per pixel for optimization 
       laipix(:) = zero
       gpppix(:) = zero
       !laieff_pix (:) = zero
       lai(:,1) = zero
       gpp_daily(:,1) = zero
       laipix(:) = SUM(lai(:,:)*veget_max(:,:),2)
       gpppix(:) = SUM(gpp_daily(:,:)*veget_max(:,:),2)
       !WRITE (numout,*) 'lai'
       !WRITE (numout,*) 'lai per pixel', laipix 
       CALL histwrite_p(hist_id_stomate, 'LAIPIX', itime, laipix, npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'NPP', itime, &
            npp_daily, npts*nvm, horipft_index)
       ! Use orphan flux so gpp is accounted for even if the PFT was removed
       CALL histwrite_p (hist_id_stomate, 'GPP', itime, &
            orphan_flux(:,:,igpp,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p(hist_id_stomate, 'GPPPIX',itime, gpppix, npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'IND', itime, &
            ind, npts*nvm, horipft_index)

       ! Calculate lai per level.  Also print out the real height of
       ! the canopy level.
       DO ivm=1,nvm
!          CALL combine_lai_levels(lai_per_level,lai_per_level_temp)
!          DO ipts=1,npts
!             lai_per_level_temp(ipts,ivm,:)=SUM(lai_per_level_temp(ipts,ivm,:))
!          ENDDO
          WRITE(var_name,'(A,I3.3)') 'LAI_PER_LEVEL_',ivm
          CALL histwrite_p (hist_id_stomate, var_name, itime, &
               lai_per_level(:,ivm,:), npts*nlevels_tot, horican_index)

          WRITE(var_name,'(A,I3.3)') 'CLEVEL_HEIGHT_',ivm
          CALL histwrite_p (hist_id_stomate, var_name, itime, &
               z_level_photo(:,ivm,:), npts*nlevels_tot, horican_index) 

       ENDDO

       CALL histwrite_p (hist_id_stomate, 'CN_IND', itime, &
            cn_ind, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'WOODMASS_IND', itime, &
            woodmass_ind, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'LEAF_M', itime, &
            biomass(:,:,ileaf,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'SAP_M_AB', itime, &
            biomass(:,:,isapabove,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'SAP_M_BE', itime, &
            biomass(:,:,isapbelow,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'HEART_M_AB', itime, &
            biomass(:,:,iheartabove,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'HEART_M_BE', itime, &
            biomass(:,:,iheartbelow,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'ROOT_M', itime, &
            biomass(:,:,iroot,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'FRUIT_M', itime, &
            biomass(:,:,ifruit,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'RESERVE_M', itime, &
            biomass(:,:,icarbres,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'LABILE_M', itime, &
            biomass(:,:,ilabile,icarbon), npts*nvm, horipft_index)

       ! Total living biomass (gC m-2)
       tot_live_biomass(:,:,:) = biomass(:,:,ileaf,:) + biomass(:,:,isapabove,:) &
            + biomass(:,:,isapbelow,:) + &
            biomass(:,:,iheartabove,:) + &
            biomass(:,:,iheartbelow,:) + &
            biomass(:,:,iroot,:) + &
            biomass(:,:,ifruit,:) + biomass(:,:,icarbres,:)

       ! Written in g C m-2
       CALL histwrite_p (hist_id_stomate, 'TOTAL_M', itime, &
            tot_live_biomass(:,:,icarbon), npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'LEAF_TURN', itime, &
            turnover_daily(:,:,ileaf,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'SAP_AB_TURN', itime, &
            turnover_daily(:,:,isapabove,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'ROOT_TURN', itime, &
            turnover_daily(:,:,iroot,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'FRUIT_TURN', itime, &
            turnover_daily(:,:,ifruit,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'LEAF_BM_LITTER', itime, &
            bm_to_litter(:,:,ileaf,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'SAP_AB_BM_LITTER', itime, &
            bm_to_litter(:,:,isapabove,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'SAP_BE_BM_LITTER', itime, &
            bm_to_litter(:,:,isapbelow,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'HEART_AB_BM_LITTER', itime, &
            bm_to_litter(:,:,iheartabove,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'HEART_BE_BM_LITTER', itime, &
            bm_to_litter(:,:,iheartbelow,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'ROOT_BM_LITTER', itime, &
            bm_to_litter(:,:,iroot,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'FRUIT_BM_LITTER', itime, &
            bm_to_litter(:,:,ifruit,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'RESERVE_BM_LITTER', itime, &
            bm_to_litter(:,:,icarbres,icarbon), npts*nvm, horipft_index)
       ! Use orphan flux so maint_resp is accounted for even if the PFT was removed
       CALL histwrite_p (hist_id_stomate, 'MAINT_RESP', itime, &
            orphan_flux(:,:,irmain,icarbon), npts*nvm, horipft_index)
       ! Use orphan flux so growth_resp is accounted for even if the PFT was removed
       CALL histwrite_p (hist_id_stomate, 'GROWTH_RESP', itime, &
            orphan_flux(:,:,irgrow,icarbon), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'AGE', itime, &
            age, npts*nvm, horipft_index)
        CALL histwrite_p (hist_id_stomate, 'DIAMETER', itime, &
            qm_dia, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'HEIGHT', itime, &
            qm_height, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'MOISTRESS', itime, &
            moiavail_week, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'VCMAX', itime, &
            vcmax, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'TURNOVER_TIME_LEAF', itime, &
            turnover_time(:,:,ileaf), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'TURNOVER_TIME_ROOT', itime, &
            turnover_time(:,:,iroot), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'TURNOVER_TIME_SAP', itime, &
            turnover_time(:,:,isapabove), npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'TURNOVER_TIME_FRUIT', itime, &
            turnover_time(:,:,ifruit), npts*nvm, horipft_index) 

       !! 16.2 Land cover and management change variables

       CALL histwrite_p (hist_id_stomate, 'CFLUX_PROD_S', itime, &
            cflux_prod_s, npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'CFLUX_PROD_M', itime, &
            cflux_prod_m, npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'CFLUX_PROD_L', itime, &
            cflux_prod_l, npts, hori_index)
       CALL histwrite_p (hist_id_stomate, 'PROD_S', itime, &
            prod_s, npts*(nshort+1), horip_ss_index)
       CALL histwrite_p (hist_id_stomate, 'PROD_M', itime, &
            prod_m, npts*(nmedium+1), horip_mm_index)
       CALL histwrite_p (hist_id_stomate, 'PROD_L', itime, &
            prod_l, npts*(nlong+1), horip_ll_index)
       CALL histwrite_p (hist_id_stomate, 'FLUX_S', itime, &
            flux_s, npts*nshort, horip_s_index)
       CALL histwrite_p (hist_id_stomate, 'FLUX_M', itime, &
            flux_m, npts*nmedium, horip_m_index)
       CALL histwrite_p (hist_id_stomate, 'FLUX_L', itime, &
            flux_l, npts*nlong, horip_l_index)
       CALL histwrite_p (hist_id_stomate, 'BM_SYNC', itime, &
            bm_sync(:,:,icarbon), npts*nvm, horipft_index) 

       ! for functional allocation, we have all of this information
       ! at every step so we can write it out.
       var_real=REAL(age_stand)
       CALL histwrite_p (hist_id_stomate, 'AGE_STAND', itime, &
            var_real, npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'MAI', itime, &
            mai, npts*nvm, horipft_index) 
       CALL histwrite_p (hist_id_stomate, 'PAI', itime, &
            pai, npts*nvm, horipft_index) 
       var_real=REAL(rotation_n)
       CALL histwrite_p (hist_id_stomate, 'ROTATION_N', itime, &
            var_real, npts*nvm, horipft_index)
       var_real=REAL(last_cut)
       CALL histwrite_p (hist_id_stomate, 'LAST_CUT', itime, &
            var_real, npts*nvm, horipft_index)
       var_real=REAL(forest_managed)
       CALL histwrite_p (hist_id_stomate, 'FOREST_MANAGED', itime, &
            var_real, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'LITTER_RAKE_FRAC', itime, &
            lrake_frac, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'SIGMA', itime, &
            sigma, npts*nvm, horipft_index)
       CALL histwrite_p (hist_id_stomate, 'GAMMA', itime, &
            gammas, npts*nvm, horipft_index)

       ! These next ones are all commented out because the variable they
       ! are writing no longer exists
!       CALL histwrite_p (hist_id_stomate, 'EXPORTED_BM', itime, &
!            exported_bm, npts*nvm, horipft_index)
!       CALL histwrite_p (hist_id_stomate, 'LAI_MAX_LYEAR', itime, &
!            lai_max_lastyear, npts*nvm, horipft_index) 
!       lai_max_lastyear(:,:) = zero
!       CALL histwrite_p (hist_id_stomate, 'STANDING_AB_BM', itime, &
!            standing_ab_bm, npts*nvm, horipft_index) 
!       CALL histwrite_p (hist_id_stomate, 'DELTAVOL', itime, &
!            deltavol, npts*nvm, horipft_index) 
!       CALL histwrite_p (hist_id_stomate, 'STANDING_AB_VOL', itime, &
!            standing_ab_vol, npts*nvm, horipft_index)
!       CALL histwrite_p (hist_id_stomate, 'HAR_VOL', itime, &
!            harvestable_vol, npts*nvm, horipft_index)
!       CALL histwrite_p (hist_id_stomate, 'HAR_VOL_INC', itime, &
!            harvestable_vol_inc, npts*nvm, horipft_index)
!       CALL histwrite_p (hist_id_stomate, 'EXPORTED_VOL', itime, &
!            exported_vol, npts*nvm, horipft_index)
!       CALL histwrite_p (hist_id_stomate, 'EXPORTED_VOL_TH', itime, &
!            exported_vol_th, npts*nvm, horipft_index)
!       CALL histwrite_p (hist_id_stomate, 'CIRC_MIN', itime, &
!            circ_min, npts*nvm, horipft_index)
!       CALL histwrite_p (hist_id_stomate, 'CIRC_MAX', itime, &
!            circ_max, npts*nvm, horipft_index)
       !             CALL histwrite_p (hist_id_stomate, 'RDI', itime, &
       !                  rdi, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC02', itime, &
!!$            circ_class_n(:,:,ic02), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC04', itime, &
!!$            circ_class_n(:,:,ic04), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC06', itime, &
!!$            circ_class_n(:,:,ic06), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC08', itime, &
!!$            circ_class_n(:,:,ic08), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC10', itime, &
!!$            circ_class_n(:,:,ic10), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC12', itime, &
!!$            circ_class_n(:,:,ic12), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC14', itime, &
!!$            circ_class_n(:,:,ic14), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC16', itime, &
!!$            circ_class_n(:,:,ic16), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC18', itime, &
!!$            circ_class_n(:,:,ic18), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC20', itime, &
!!$            circ_class_n(:,:,ic20), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC222', itime, &
!!$            circ_class_n(:,:,ic222), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CCPO', itime, &
!!$            circ_class_n(:,:,icpo), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CCSW', itime, &
!!$            circ_class_n(:,:,icsw), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CCMW', itime, &
!!$            circ_class_n(:,:,icmw), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CCLW', itime, &
!!$            circ_class_n(:,:,iclw), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC02_BT', itime, &
!!$            circ_class_n_bt(:,:,ic02), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC04_BT', itime, &
!!$            circ_class_n_bt(:,:,ic04), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC06_BT', itime, &
!!$            circ_class_n_bt(:,:,ic06), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC08_BT', itime, &
!!$            circ_class_n_bt(:,:,ic08), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC10_BT', itime, &
!!$            circ_class_n_bt(:,:,ic10), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC12_BT', itime, &
!!$            circ_class_n_bt(:,:,ic12), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC14_BT', itime, &
!!$            circ_class_n_bt(:,:,ic14), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC16_BT', itime, &
!!$            circ_class_n_bt(:,:,ic16), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC18_BT', itime, &
!!$            circ_class_n_bt(:,:,ic18), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC20_BT', itime, &
!!$            circ_class_n_bt(:,:,ic20), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CC222_BT', itime, &
!!$            circ_class_n_bt(:,:,ic222), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CCPO_BT', itime, &
!!$            circ_class_n_bt(:,:,icpo), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CCSW_BT', itime, &
!!$            circ_class_n_bt(:,:,icsw), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CCMW_BT', itime, &
!!$            circ_class_n_bt(:,:,icmw), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CCLW_BT', itime, &
!!$            circ_class_n_bt(:,:,iclw), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'IND_BT', itime, &
!!$            ind_bt, npts*nvm, horipft_index)
       !++++++ end variables which no longer exist
       !-----

       !! 16.3 Calculated variables

       ! Soil and litter carbon (gC m-2)
       tot_soil_carb(:,:) = zero
       tot_litter_carb(:,:) = zero
       DO j=1,nvm

          tot_litter_carb(:,j) = tot_litter_carb(:,j) + &
               (litter(:,istructural,j,iabove,icarbon) + &
               litter(:,imetabolic,j,iabove,icarbon) + &
               litter(:,istructural,j,ibelow,icarbon) + &
               litter(:,imetabolic,j,ibelow,icarbon) + &
               litter(:,iwoody,j,iabove,icarbon) + &
               litter(:,iwoody,j,ibelow,icarbon))
          tot_soil_carb(:,j) = tot_soil_carb(:,j) + (carbon(:,iactive,j) + &
               carbon(:,islow,j)+  carbon(:,ipassive,j))

       ENDDO
       tot_litter_soil_carb(:,:) = tot_litter_carb(:,:) + tot_soil_carb(:,:)

       CALL xios_orchidee_send_field("TOTAL_SOIL_CARB",tot_litter_soil_carb)
       CALL xios_orchidee_send_field("cLitter",SUM(tot_litter_carb*veget_max,dim=2)/1e3*contfrac)
       CALL xios_orchidee_send_field("cSoil",SUM(tot_soil_carb*veget_max,dim=2)/1e3*contfrac)

!!$       CALL histwrite_p (hist_id_stomate, 'TOTAL_SOIL_CARB', itime, &
!!$            tot_litter_soil_carb, npts*nvm, horipft_index)

       ! Write output file for IPCC
       IF ( hist_id_stomate_IPCC > 0 ) THEN

          vartmp(:)=SUM(tot_litter_carb*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cLitter", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(tot_soil_carb*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cSoil", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=(prod_s_total + prod_m_total + prod_l_total)/1e3
          CALL histwrite_p (hist_id_stomate_IPCC, "cProduct", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(lai*veget_max,dim=2)*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "lai", itime, &
               vartmp, npts, hori_index)
          ! Use orphan flux so gpp is accounted for even if the PFT was removed
          vartmp(:)=SUM(orphan_flux(:,:,igpp,icarbon)*orphan_flux(:,:,ivegold,icarbon),dim=2)/&
               1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "gpp", itime, &
               vartmp, npts, hori_index)
          ! Use orphan fluxes so autotrophic respiration is accounted for even if the PFT was removed
          vartmp(:)=SUM(orphan_flux(:,:,irgrow,icarbon)*orphan_flux(:,:,ivegold,icarbon),dim=2)/&
               1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "rGrowth", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(orphan_flux(:,:,irmain,icarbon)*orphan_flux(:,:,ivegold,icarbon),dim=2)/&
               1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "rMaint", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM((orphan_flux(:,:,irmain,icarbon)+orphan_flux(:,:,irgrow,icarbon))*&
               orphan_flux(:,:,ivegold,icarbon),dim=2)/1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "ra", itime, &
               vartmp, npts, hori_index)
          ! Use orphan fluxes so npp is accounted for even if the PFT was removed
          vartmp(:)=SUM(orphan_flux(:,:,inpp,icarbon)*orphan_flux(:,:,ivegold,icarbon),dim=2)/&
               1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "npp", itime, &
               vartmp, npts, hori_index)
          ! Use orphan fluxes so heterotrophic respiration is accounted for even if the PFT was removed
          vartmp(:)=SUM(orphan_flux(:,:,irhet,icarbon)*orphan_flux(:,:,ivegold,icarbon),dim=2)/&
               1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "rh", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(co2_fire*veget_max,dim=2)/1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "fFire", itime, &
               vartmp, npts, hori_index)
        
          vartmp(:)=SUM(SUM(soilcarbon_input,dim=2)*veget_max,dim=2)&
               /1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "fLitterSoil", itime, &
               vartmp, npts, hori_index)
          ! The biomass pools are updated during land cover change
          vartmp(:)=SUM(biomass(:,:,ileaf,icarbon)*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cLeaf", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM((biomass(:,:,isapabove,icarbon)+&
               biomass(:,:,iheartabove,icarbon))*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cWood", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(( biomass(:,:,iroot,icarbon) + biomass(:,:,isapbelow,icarbon) &
               + biomass(:,:,iheartbelow,icarbon) ) &
               *veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cRoot", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(( biomass(:,:,icarbres,icarbon) + &
               biomass(:,:,ifruit,icarbon))*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cMisc", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM((litter(:,istructural,:,iabove,icarbon)+&
               litter(:,imetabolic,:,iabove,icarbon))*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cLitterAbove", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM((litter(:,istructural,:,ibelow,icarbon)+&
               litter(:,imetabolic,:,ibelow,icarbon))*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cLitterBelow", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(carbon(:,iactive,:)*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cSoilFast", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(carbon(:,islow,:)*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cSoilMedium", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(carbon(:,ipassive,:)*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cSoilSlow", itime, &
               vartmp, npts, hori_index)
          DO j=1,nvm
             histvar(:,j)=veget_max(:,j)*contfrac(:)*100
          ENDDO
          CALL histwrite_p (hist_id_stomate_IPCC, "landCoverFrac", itime, &
               histvar, npts*nvm, horipft_index)
          vartmp(:)=zero
          DO j = 2,nvm
             IF (is_deciduous(j)) THEN
                vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
             ENDIF
          ENDDO
          CALL histwrite_p (hist_id_stomate_IPCC, "treeFracPrimDec", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=zero
          DO j = 2,nvm
             IF (is_evergreen(j)) THEN
                vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
             ENDIF
          ENDDO
          CALL histwrite_p (hist_id_stomate_IPCC, "treeFracPrimEver", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=zero
          DO j = 2,nvm
             IF ( .NOT.(is_c4(j)) ) THEN
                vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
             ENDIF
          ENDDO
          CALL histwrite_p (hist_id_stomate_IPCC, "c3PftFrac", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=zero
          DO j = 2,nvm
             IF ( is_c4(j) ) THEN
                vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
             ENDIF
          ENDDO
          CALL histwrite_p (hist_id_stomate_IPCC, "c4PftFrac", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(bm_alloc(:,:,ileaf,icarbon)*veget_max,dim=2)/1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "nppLeaf", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(bm_alloc(:,:,isapabove,icarbon)*veget_max,dim=2)/1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "nppWood", itime, &
               vartmp, npts, hori_index)
          vartmp(:)=SUM(( bm_alloc(:,:,isapbelow,icarbon) + bm_alloc(:,:,iroot,icarbon) )*veget_max,dim=2)/&
               1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "nppRoot", itime, &
               vartmp, npts, hori_index)
          CALL histwrite_p (hist_id_stomate_IPCC, 'RESOLUTION_X', itime, &
               resolution(:,1), npts, hori_index)
          CALL histwrite_p (hist_id_stomate_IPCC, 'RESOLUTION_Y', itime, &
               resolution(:,2), npts, hori_index)
          CALL histwrite_p (hist_id_stomate_IPCC, 'CONTFRAC', itime, &
               contfrac(:), npts, hori_index)

          ! Total living biomass (gC m-2)
          tot_live_biomass(:,:,:) = biomass(:,:,ileaf,:) + biomass(:,:,isapabove,:) &
               + biomass(:,:,isapbelow,:) + &
               biomass(:,:,iheartabove,:) + &
               biomass(:,:,iheartbelow,:) + &
               biomass(:,:,iroot,:) + &
               biomass(:,:,ifruit,:) + biomass(:,:,icarbres,:)

          CALL xios_orchidee_send_field("TOTAL_M",tot_live_biomass)
          CALL xios_orchidee_send_field("cVeg",SUM(tot_live_biomass(:,:,icarbon)*veget_max,dim=2)/1e3*contfrac)

          ! Written in kg C m-2, note the division by 1000 and multiplication by veget_max 
          vartmp(:)=SUM(tot_live_biomass(:,:,icarbon)*veget_max,dim=2)/1e3*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cVeg", itime, &
               vartmp, npts, hori_index)

          ! Written in g C m-2
          CALL histwrite_p (hist_id_stomate, 'TOTAL_M', itime, &
               tot_live_biomass(:,:,icarbon), npts*nvm, horipft_index) 

          ! Ratio of above and belowground woody biomass (shoot_ratio)
          shoot_ratio(:,:) = zero
          WHERE ( (biomass(:,:,isapabove,icarbon) + biomass(:,:,iheartabove,icarbon) & 
               + biomass(:,:,isapbelow,icarbon) + biomass(:,:,iheartbelow,icarbon)) > min_stomate )

             shoot_ratio(:,:) = (biomass(:,:,isapabove,icarbon) + &
                  biomass(:,:,iheartabove,icarbon))/ &
                  (biomass(:,:,isapabove,icarbon) + biomass(:,:,iheartabove,icarbon) + &
                  biomass(:,:,isapbelow,icarbon) + biomass(:,:,iheartbelow,icarbon))

          END WHERE

          ! Aboveground woody biomass (gC m-2)
          woody_biomass_above(:,:) = biomass(:,:,isapabove,icarbon) + &
               biomass(:,:,iheartabove,icarbon) + &
               (biomass(:,:,icarbres,icarbon) + biomass(:,:,ilabile,icarbon)) * &
               shoot_ratio(:,:)

          ! Belowground woody biomass (gC m-2)
          woody_biomass_below(:,:) = biomass(:,:,isapbelow,icarbon) + &
               biomass(:,:,iheartbelow,icarbon) + &
               (biomass(:,:,icarbres,icarbon) + biomass(:,:,ilabile,icarbon)) * &
               (1. - shoot_ratio(:,:))

          WHERE (.NOT. PFTpresent(:,:))
             woody_biomass_above(:,:) = zero
             woody_biomass_below(:,:) = zero
          ENDWHERE

          !+++CHECK+++
          ! The trunk version of code calculates a series of derived fluxes i.e.
          ! ::wood_loss_ab, wood_loss_be(:,:), ::fHarvest, ::nbp
          ! Several of these fluxes make use of ::exported_bm (gC m-2). In this branch
          ! ::exported_bm was replaced by ::harvest_pool (gC). The change in variable
          ! came with a change in units. Calculating the harvest biomass in gC m-2
          ! requires adding a new dimension to this variable. Given that the variable
          ! already has six dimension (to store the activity to lead to the harvesting)
          ! it was decided not to add surface area unless really needed. Consequently 
          ! the aforementioned variables can not be calculated for the moment. Rather
          ! than having a wrong calculation in the code it was decided to delete these 
          ! variables. The way they have to be calculated can be found in previous 
          ! versions (r1630 and before) of this branch. Please note that ::exported_bm
          ! will have to be replaced by ::harvest_pool and that the units of 
          ! ::harvest_pool will need to be converted first.
          
          ! Total turnover (gC m-2 dt_slow-1)
          tot_turnover(:,:,:) = turnover_daily(:,:,ileaf,:) + &
               turnover_daily(:,:,isapabove,:) + &
               turnover_daily(:,:,isapbelow,:) + turnover_daily(:,:,iheartabove,:) + &
               turnover_daily(:,:,iheartbelow,:) + turnover_daily(:,:,iroot,:) + &
               turnover_daily(:,:,ifruit,:) + turnover_daily(:,:,icarbres,:) + &
               turnover_daily(:,:,ilabile,:)
          CALL xios_orchidee_send_field("TOTAL_TURN",tot_turnover)
          CALL histwrite_p (hist_id_stomate, 'TOTAL_TURN', itime, &
               tot_turnover(:,:,icarbon), npts*nvm, horipft_index)

          ! Total biomass to litter (gC m-2 dt_slow-1)
          tot_bm_to_litter(:,:,:) = bm_to_litter(:,:,ileaf,:) + bm_to_litter(:,:,isapabove,:) + &
               bm_to_litter(:,:,isapbelow,:) + bm_to_litter(:,:,iheartbelow,:) + &
               bm_to_litter(:,:,iheartabove,:) + bm_to_litter(:,:,iroot,:) + &
               bm_to_litter(:,:,ifruit,:) + bm_to_litter(:,:,icarbres,:)

          CALL xios_orchidee_send_field("TOTAL_BM_LITTER",tot_bm_to_litter(:,:,icarbon))
          CALL xios_orchidee_send_field("fVegLitter",SUM((tot_bm_to_litter(:,:,icarbon) + tot_turnover(:,:,icarbon))*&
               veget_max,dim=2)/1e3/one_day*contfrac)

          ! Written gC m-2 dt_slow-1
          CALL histwrite_p (hist_id_stomate, 'TOTAL_BM_LITTER', itime, &
               tot_bm_to_litter(:,:,icarbon), npts*nvm, horipft_index)

          ! Written in kg C m-2 s-1, note the division by 1000 and one day
          ! and multiplication by veget_max 
          vartmp(:)=SUM((tot_bm_to_litter(:,:,icarbon) + &
               tot_turnover(:,:,icarbon))*veget_max,dim=2)/1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "fVegLitter", itime, &
               vartmp, npts, hori_index)

          ! Total carbon mass kgC m-2 s-1, note the division by 1000 and one day
          ! and the multiplication with veget_max
          carb_mass_variation(:)=-carb_mass_total(:)
          carb_mass_total(:)=SUM((tot_live_biomass(:,:,icarbon)+tot_litter_carb+&
               tot_soil_carb)*veget_max,dim=2) + &
               (prod_s_total + prod_m_total + prod_l_total)
          carb_mass_variation(:)=carb_mass_total(:)+carb_mass_variation(:)
          CALL xios_orchidee_send_field("cMassVariation",carb_mass_variation/1e3/one_day*contfrac)
          vartmp(:)=carb_mass_variation/1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "cMassVariation", itime, &
               vartmp, npts, hori_index)
       
          vartmp(:)=cflux_prod_total/1e3/one_day*contfrac
          CALL histwrite_p (hist_id_stomate_IPCC, "fLuc", itime, &
               vartmp, npts, hori_index)
          !+++++++++++
          
       ENDIF

    ELSE

       ! This is the old version of Stomate_lpj, starting from when Didier created
       ! the branch and then updated a little when a few routines merged. While
       ! developing we tried to keep this working but at one point too many changes
       ! were introduced. There shouldn't be any big conflicts it would probably take
       ! a couple of days before this is working again. This version uses the
       ! resource based allocation and the DGVM should still work. However, there is 
       ! NO functional allocation, NO new albedo, NO forestry, NO age classes, 
       ! NO hydraulic architecture, NO multilayer energy-budget, NO analytic 
       ! photosynthese, NO mass balance closure (not guaranteed) and NO improved 
       ! consistency for veget_max, lai, height, ... Why would you want to use that?

!!$    !+++CHECK++
!!$    Declare this variables in the variable list when old forestry needs to be re-animated
!!$    Note that some variables may be missing but it is a good start
!!$    ! Old forestry parameters
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: av_circ                 !! Average circumference of trees in the stand (m)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: standing_ab_bm          !! Standing aboveground biomass (gC/m2)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: deltavol                !! Annual aboveground volume increment 
!!$                                                                                !! (m3/ha/yr)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: standing_ab_vol         !! Standing aboveground volume (m3/ha)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: exported_bm             !! Biomass exported (gC/m2)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: exported_vol            !! Volume exported (m3/m2) -- this year
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: exported_vol_th         !! Volume exported for thinnings 
!!$                                                                                !! (clear-cuts not included) (m3/m2) 
!!$                                                                                !! -- this year
!!$    REAL(r_std), DIMENSION(npts,nvm,ncirc)           :: circ_class_n_bt         !! Number of trees in each circumference 
!!$                                                                                !! class (0-2.2+ and poles-large wood)
!!$                                                                                !! (bt for before thinning)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: circ_lim                !! Limit circumference (m, trees larger than 
!!$                                                                                !! circ_lim are not affected by thinnings)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: circ_min                !! Smallest circumference (m)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: circ_max                !! Largest circumference (m)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: harvestable_vol         !! Harvestable volume (diameter > 0.175 m
!!$                                                                                !! without branches) (m3/ha)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: harvestable_vol_inc     !! Yearly increment in harvestable volume 
!!$                                                                                !! (diameter > 0.175 m without branches) 
!!$                                                                                !! (m3/ha/yr)
!!$    REAL(r_std), DIMENSION(npts,nvm)                 :: ind_bt                  !! Density of individuals before thinning 
!!$                                                                                !! @tex $(m^{-2})$ @endtex of nat/agri 
!!$                                                                                !! ground
!!$    !+++++++++ 

!!$       !! 1. Initializations
!!$
!!$       !! 1.1 Initialize variables to zero
!!$       co2_to_bm(:,:) = zero
!!$       co2_fire(:,:) = zero
!!$       npp_daily(:,:) = zero
!!$       resp_maint(:,:) = zero
!!$       resp_growth(:,:) = zero
!!$       bm_to_litter(:,:,:,:) = zero
!!$       cn_ind(:,:) = zero
!!$       woodmass_ind(:,:) = zero
!!$       turnover_daily(:,:,:,:) = zero
!!$
!!$       !! 1.2  Initialize variables to veget_max
!!$       veget_max_old(:,:) = veget_max(:,:)
!!$
!!$       ! 1.25 Correct management status if waiting for clearcut (management only changes after a clearcut, except if it's to
!!$       ! become unmanaged)
!!$       IF ( control%forest_management ) THEN
!!$
!!$          DO j = 2,nvm
!!$             DO i = 1,npts
!!$                IF ( (forest_managed(i,j) /= forest_managed_lastyear(i,j)) .AND. (forest_managed_lastyear(i,j) > 0) .AND. &
!!$                     & (forest_managed(i,j) > 0) .AND.(age_stand(i,j) > 1) ) THEN
!!$                   forest_managed(i,j) = forest_managed_lastyear(i,j)
!!$                ENDIF
!!$             ENDDO
!!$          ENDDO
!!$          IF (EndOfYear) forest_managed_lastyear(:,:) = forest_managed(:,:)
!!$
!!$       END IF ! control%forest_management
!!$
!!$       !! 1.3 Calculate some vegetation characteristics
!!$
!!$       !! 1.3.1 Calculate some vegetation characteristics 
!!$       !        Calculate cn_ind (individual crown mass) and individual height from
!!$       !        state variables if running DGVM or dynamic mortality in static cover mode
!!$       !??        Explain (maybe in the header once) why you mulitply with veget_max in the DGVM
!!$       !??        and why you don't multiply with veget_max in stomate.
!!$       IF ( control%ok_dgvm .OR. .NOT.control%ok_constant_mortality) THEN
!!$          IF(control%ok_dgvm) THEN
!!$             WHERE (ind(:,:).GT.min_stomate)
!!$                woodmass_ind(:,:) = &
!!$                     ((biomass(:,:,isapabove,icarbon)+biomass(:,:,isapbelow,icarbon) &
!!$                     +biomass(:,:,iheartabove,icarbon)+biomass(:,:,iheartbelow,icarbon)) & 
!!$                     *veget_max(:,:))/ind(:,:)
!!$             ENDWHERE
!!$          ELSE
!!$             WHERE (ind(:,:).GT.min_stomate)
!!$                woodmass_ind(:,:) = &
!!$                     (biomass(:,:,isapabove,icarbon)+biomass(:,:,isapbelow,icarbon) &
!!$                     +biomass(:,:,iheartabove,icarbon)+biomass(:,:,iheartbelow,icarbon))/ind(:,:)
!!$             ENDWHERE
!!$          ENDIF
!!$
!!$          CALL crown_diagnostic (npts,  PFTpresent, &
!!$               ind, biomass, woodmass_ind, &
!!$               veget_max, cn_ind, height, &
!!$               forest_managed)
!!$       ENDIF
!!$
!!$       !! 1.3.2 Prescribe characteristics if the vegetation is not dynamic
!!$       !        IF the DGVM is not activated, the density of individuals and their crown
!!$       !        areas don't matter, but they should be defined for the case we switch on
!!$       !        the DGVM afterwards. At the first call, if the DGVM is not activated, 
!!$       !        impose a minimum biomass for prescribed PFTs and declare them present.
!!$       CALL prescribe_diagnostic (npts, &
!!$            veget_max, dt_days, PFTpresent, everywhere, when_growthinit, &
!!$            biomass, leaf_frac, ind, cn_ind, co2_to_bm)
!!$
!!$
!!$       !! 2. Climatic constraints for PFT presence and regenerativeness
!!$
!!$       !   Call this even when DGVM is not activated so that "adapted" and "regenerate"
!!$       !   are kept up to date for the moment when the DGVM is activated.
!!$       CALL constraints (npts, dt_days, &
!!$            t2m_month, t2m_min_daily,when_growthinit, &
!!$            adapted, regenerate)
!!$
!!$
!!$       !! 3. Determine introduction and elimination of PTS based on climate criteria
!!$
!!$       IF ( control%ok_dgvm ) THEN
!!$
!!$          !! 3.1 Calculate introduction and elimination
!!$          CALL pftinout (npts, dt_days, adapted, regenerate, &
!!$               neighbours, veget_max, &
!!$               biomass, ind, cn_ind, age, leaf_frac, npp_longterm, lm_lastyearmax, senescence, &
!!$               PFTpresent, everywhere, when_growthinit, need_adjacent, RIP_time, &
!!$               co2_to_bm, &
!!$               avail_tree, avail_grass)
!!$
!!$          !! 3.2 Reset attributes for eliminated PFTs.
!!$          !     This also kills PFTs that had 0 leafmass during the last year. The message
!!$          !     "... after pftinout" is misleading in this case.
!!$          CALL kill_diagnostic (npts, 'pftinout  ', lm_lastyearmax, &
!!$               ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
!!$               age, leaf_age, leaf_frac, npp_longterm, &
!!$               when_growthinit, everywhere, veget_max, bm_to_litter)
!!$
!!$
!!$          !! 3.3 Calculate new crown area and diameter 
!!$          !      Calculate new crown area, diameter and maximum vegetation cover**[No longer used in the subroutine]
!!$          !      unsure whether this is really required
!!$          !      - in theory this could ONLY be done at the END of stomate_lpj
!!$          !      calculate woodmass of individual tree
!!$          WHERE ((ind(:,:).GT.min_stomate))
!!$             WHERE  ( veget_max(:,:) .GT. min_stomate)
!!$                woodmass_ind(:,:) = &
!!$                     ((biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
!!$                     + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon))*veget_max(:,:))/ind(:,:)
!!$             ELSEWHERE
!!$                woodmass_ind(:,:) =(biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
!!$                     + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon))/ind(:,:)
!!$             ENDWHERE
!!$
!!$          ENDWHERE
!!$
!!$          ! Calculate crown area and diameter for all PFTs (including the newly established)
!!$          CALL crown_diagnostic (npts, PFTpresent, &
!!$               ind, biomass, woodmass_ind, &
!!$               veget_max, cn_ind, height, &
!!$               forest_managed)
!!$
!!$       ENDIF
!!$
!!$       !! 4. Phenology
!!$
!!$       !! 4.1 Write values to history file
!!$       !      Current values for ::when_growthinit 
!!$       CALL histwrite_p (hist_id_stomate, 'WHEN_GROWTHINIT', itime, when_growthinit, npts*nvm, horipft_index)
!!$
!!$       ! Set and write values for ::PFTpresent
!!$       WHERE(PFTpresent)
!!$          histvar=un
!!$       ELSEWHERE
!!$          histvar=zero
!!$       ENDWHERE
!!$       CALL histwrite_p (hist_id_stomate, 'PFTPRESENT', itime, histvar, npts*nvm, horipft_index)
!!$
!!$       ! Set and write values for gdd_midwinter
!!$       WHERE(gdd_midwinter.EQ.undef)
!!$          histvar=val_exp
!!$       ELSEWHERE
!!$          histvar=gdd_midwinter
!!$       ENDWHERE
!!$       CALL histwrite_p (hist_id_stomate, 'GDD_MIDWINTER', itime, histvar, npts*nvm, horipft_index)
!!$
!!$       ! Set and write values for gdd_m5_dormance
!!$       WHERE(gdd_m5_dormance.EQ.undef)
!!$          histvar=val_exp
!!$       ELSEWHERE
!!$          histvar=gdd_m5_dormance
!!$       ENDWHERE
!!$       CALL histwrite (hist_id_stomate, 'GDD_M5_DORMANCE', itime, histvar, npts*nvm, horipft_index)
!!$
!!$       ! Set and write values for ncd_dormance
!!$       WHERE(ncd_dormance.EQ.undef)
!!$          histvar=val_exp
!!$       ELSEWHERE
!!$          histvar=ncd_dormance
!!$       ENDWHERE
!!$       CALL histwrite_p (hist_id_stomate, 'NCD_DORMANCE', itime, histvar, npts*nvm, horipft_index)
!!$
!!$       !! 4.2 Calculate phenology
!!$       CALL phenology_diagnostic (npts, dt_days, PFTpresent, &
!!$            veget_max, &
!!$            tlong_ref, t2m_month, t2m_week, gpp_daily, &
!!$            maxmoiavail_lastyear, minmoiavail_lastyear, &
!!$            moiavail_month, moiavail_week, &
!!$            gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
!!$            senescence, time_hum_min, &
!!$            biomass, leaf_frac, leaf_age, &
!!$            when_growthinit, co2_to_bm)
!!$
!!$       !! 5. Allocate C to different plant parts
!!$       CALL growth_res_lim (npts, dt_days, lai, veget_max, & 
!!$            PFTpresent, senescence, when_growthinit, moiavail_week, & 
!!$            soilhum_month, tsoil_month, gpp_daily, resp_maint_part, & 
!!$            resp_maint, resp_growth, npp_daily, biomass, & 
!!$            age, leaf_age, leaf_frac)
!!$
!!$!       CALL alloc (npts, dt_days, &
!!$!            lai, veget_max, senescence, when_growthinit, &
!!$!            moiavail_week, tsoil_month, soilhum_month, &
!!$!            biomass, age, leaf_age, leaf_frac, rprof, f_alloc, &
!!$!            lai_max_calc, forest_managed, age_stand, last_cut)
!!$
!!$       !! 6. NPP, maintenance and growth respiration
!!$
!!$       !! 6.1 Calculate NPP and respiration terms
!!$!       CALL npp_calc (npts, dt_days, &
!!$!            PFTpresent, &
!!$!            tlong_ref, t2m_daily, tsoil_daily, lai, rprof, &
!!$!            gpp_daily, f_alloc, bm_alloc, resp_maint_part,&
!!$!            biomass, leaf_age, leaf_frac, age, &
!!$!            resp_maint, resp_growth, npp_daily)
!!$
!!$
!!$       IF ( control%forest_management ) THEN
!!$
!!$          ! We have made extensive changes to the flow of the forestry
!!$          ! module.  As the version of the forestry module in this
!!$          ! branch had never been tested anyway, it doesn't make
!!$          ! sense to maintain any backwards compatibility.  If one
!!$          ! wants to do forestry, the new allocation scheme already
!!$          ! has circumference classes, so the old way of creating
!!$          ! circumference classes from a big leaf canopy does not
!!$          ! make any sense.  In addition, forestry should introduce
!!$          ! gaps into the canopy, but the old albedo scheme could not
!!$          ! handle gaps.  So its better to use the new allocation scheme.
!!$
!!$          STOP 'Cannot run forestry with the old allocation.'
!!$!          CALL forestry (npts, biomass, lai, &
!!$!               ind, circ, EndOfYear, & 
!!$!               lai_max_calc, circ_class, bm_to_litter, &
!!$!               biomass_lastyear, lai_max_lastyear, when_growthinit, leaf_frac, &
!!$!               veget_max, sigma, gamma, age_stand, &
!!$!               rotation_n, last_cut, av_circ, ba, &
!!$!               dom_height, av_height, &
!!$!               med_height, q1_height, q3_height, &
!!$!               standing_ab_bm, deltavol, standing_ab_vol, & 
!!$!               harvestable_vol, harvestable_vol_inc, exported_bm, &
!!$!               exported_vol, exported_vol_th, & 
!!$!               circ_lim, circ_min, circ_max, circ_class_n, circ_class_n_bt, &
!!$!               forest_managed, ind_bt, rdi)
!!$
!!$       END IF ! control%forest_management
!!$
!!$       !! 6.2 Kill slow growing PFTs in DGVM or STOMATE with constant mortality
!!$       IF ( control%ok_dgvm .OR. .NOT.control%ok_constant_mortality) THEN
!!$          CALL kill_diagnostic (npts, 'npp       ', lm_lastyearmax,  &
!!$               ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
!!$               age, leaf_age, leaf_frac, npp_longterm, &
!!$               when_growthinit, everywhere, veget_max, bm_to_litter)
!!$
!!$          !! 6.2.1 Update wood biomass      
!!$          !        For the DGVM
!!$          IF(control%ok_dgvm) THEN
!!$             WHERE (ind(:,:).GT.min_stomate)
!!$                woodmass_ind(:,:) = &
!!$                     ((biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
!!$                     + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon)) & 
!!$                     *veget_max(:,:))/ind(:,:)
!!$             ENDWHERE
!!$
!!$             ! For all pixels with individuals
!!$          ELSE
!!$             WHERE (ind(:,:).GT.min_stomate)
!!$                woodmass_ind(:,:) = &
!!$                     (biomass(:,:,isapabove,icarbon) + biomass(:,:,isapbelow,icarbon) &
!!$                     + biomass(:,:,iheartabove,icarbon) + biomass(:,:,iheartbelow,icarbon))/ind(:,:)
!!$             ENDWHERE
!!$          ENDIF ! control%ok_dgvm
!!$
!!$          !! 6.2.2 New crown area and maximum vegetation cover after growth
!!$          CALL crown_diagnostic (npts, PFTpresent, &
!!$               ind, biomass, woodmass_ind,&
!!$               veget_max, cn_ind, height, &
!!$               forest_managed)
!!$
!!$       ENDIF ! control%ok_dgvm
!!$
!!$       !! 7. fire
!!$
!!$       !! 7.1. Burn PFTs
!!$       CALL fire (npts, dt_days, litterpart, &
!!$            litterhum_daily, t2m_daily, lignin_struc, lignin_wood, veget_max, &
!!$            fireindex, firelitter, biomass, ind, &
!!$            litter, dead_leaves, bm_to_litter, black_carbon, &
!!$            co2_fire, MatrixA)
!!$
!!$       !! 7.2 Kill PFTs in DGVM
!!$       IF ( control%ok_dgvm ) THEN
!!$
!!$          ! reset attributes for eliminated PFTs
!!$          CALL kill_diagnostic (npts, 'fire      ', lm_lastyearmax, &
!!$               ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
!!$               age, leaf_age, leaf_frac, npp_longterm, &
!!$               when_growthinit, everywhere, veget_max, bm_to_litter)
!!$
!!$       ENDIF ! control%ok_dgvm
!!$
!!$       !! 8. Tree mortality
!!$
!!$       ! Does not depend on age, therefore does not change crown area.
!!$       CALL gap_diagnostic (npts, dt_days, &
!!$            npp_longterm, turnover_longterm, lm_lastyearmax, &
!!$            PFTpresent, biomass, ind, bm_to_litter, mortality , &
!!$            forest_managed)
!!$
!!$       IF ( control%ok_dgvm ) THEN
!!$
!!$          ! reset attributes for eliminated PFTs
!!$          CALL kill_diagnostic (npts, 'gap       ', lm_lastyearmax, &
!!$               ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
!!$               age, leaf_age, leaf_frac, npp_longterm, &
!!$               when_growthinit, everywhere, veget_max, bm_to_litter)
!!$
!!$       ENDIF
!!$
!!$       !! 9. Calculate vcmax and photosynthesis temperatures
!!$
!!$       CALL vmax_diagnostic (npts, dt_days, &
!!$            leaf_age, leaf_frac, &
!!$            forest_managed, age_stand, &
!!$            vcmax)
!!$
!!$       CALL assim_temp (npts, tlong_ref, t2m_month, &
!!$            t_photo_min, t_photo_opt, t_photo_max)
!!$
!!$       !! 10. Leaf senescence, new lai and other turnover processes
!!$
!!$       CALL turnover_diagnostic (npts, dt_days, PFTpresent, &
!!$            herbivores, &
!!$            maxmoiavail_lastyear, minmoiavail_lastyear, &
!!$            moiavail_week,  moiavail_month,tlong_ref, t2m_month, t2m_week, veget_max, &
!!$            gdd_from_growthinit, leaf_age, leaf_frac, age, lai, biomass, &
!!$            turnover_daily, senescence, turnover_time, &
!!$            forest_managed)
!!$
!!$       !! 11. Light competition
!!$
!!$       !! If not using constant mortality then kill with light competition
!!$       !    IF ( control%ok_dgvm .OR. .NOT.(control%ok_constant_mortality) ) THEN
!!$       IF ( control%ok_dgvm ) THEN
!!$
!!$          !! 11.1 Light competition
!!$          CALL light (npts, dt_days, &
!!$               veget_max, fpc_max, PFTpresent, cn_ind, lai, maxfpc_lastyear, &
!!$               lm_lastyearmax, ind, biomass, veget_lastlight, bm_to_litter, mortality)
!!$
!!$          !! 11.2 Reset attributes for eliminated PFTs
!!$          CALL kill_diagnostic (npts, 'light     ', lm_lastyearmax, &
!!$               ind, PFTpresent, cn_ind, biomass, senescence, RIP_time, &
!!$               age, leaf_age, leaf_frac, npp_longterm, &
!!$               when_growthinit, everywhere, veget_max, bm_to_litter)
!!$
!!$       ENDIF
!!$
!!$
!!$       !! 12. Establishment of saplings
!!$
!!$       IF ( control%ok_dgvm .OR. .NOT.control%ok_constant_mortality ) THEN
!!$
!!$          !! 12.1 Establish new plants
!!$          CALL establish_diagnostic (npts, dt_days, PFTpresent, regenerate, &
!!$               neighbours, resolution, need_adjacent, herbivores, &
!!$               precip_lastyear, gdd0_lastyear, lm_lastyearmax, &
!!$               cn_ind, lai, avail_tree, avail_grass, npp_longterm, &
!!$               leaf_age, leaf_frac, &
!!$               ind, biomass, age, everywhere, co2_to_bm, veget_max, woodmass_ind)
!!$
!!$          !! 12.2 Calculate new crown area (and maximum vegetation cover)
!!$          CALL crown_diagnostic (npts, PFTpresent, &
!!$               ind, biomass, woodmass_ind, &
!!$               veget_max, cn_ind, height, &
!!$               forest_managed)
!!$
!!$       ENDIF
!!$
!!$       !! 13. Calculate final LAI and vegetation cover
!!$
!!$       CALL cover (npts, cn_ind, ind, biomass, &
!!$            veget_max, veget_max_old, lai, &
!!$            litter, carbon, turnover_daily, bm_to_litter, &
!!$            lignin_struc, lignin_wood)
!!$
!!$       !! 14. Update litter pools to account for harvest
!!$
!!$       ! the whole litter stuff:
!!$       !    litter update, lignin content, PFT parts, litter decay, 
!!$       !    litter heterotrophic respiration, dead leaf soil cover.
!!$       !    No vertical discretisation in the soil for litter decay.\n
!!$       ! added by shilong for harvest
!!$       IF (control%ok_agricultural_harvest) THEN
!!$          CALL crop_harvest(npts, dt_days, veget_max, turnover_daily, &
!!$               harvest_pool, resolution)
!!$       ENDIF
!!$
!!$       !! 15. Land cover change
!!$
!!$       !shilong adde turnover_daily
!!$       IF(EndOfYear) THEN
!!$          IF (control%ok_land_cover_change) THEN
!!$             CALL lcchange_main (npts, dt_days, veget_max, veget_max_new, &
!!$                  biomass, ind, age, PFTpresent, senescence, when_growthinit, everywhere, &
!!$                  co2_to_bm, bm_to_litter, turnover_daily, cn_ind,flux_m,flux_l, &
!!$!               prod10,prod100,prod10_total,prod100_total,&
!!$!               convflux,cflux_prod_total,cflux_prod10,cflux_prod100,leaf_frac,&
!!$                  prod_m,prod_l,cflux_prod_s,cflux_prod_m,cflux_prod_l,leaf_frac,&
!!$                  npp_longterm, lm_lastyearmax, litter, carbon , &
!!$                  lignin_struc, lignin_wood)
!!$          ENDIF
!!$       ENDIF
!!$       !MM déplacement pour initialisation correcte des grandeurs cumulées :
!!$       cflux_prod_total(:) = cflux_prod_s(:) + cflux_prod_m(:) + cflux_prod_l(:)
!!$       prod_m_total(:) = SUM(prod_m(:,:),dim=2)
!!$       prod_l_total(:) = SUM(prod_l(:,:),dim=2)
!!$       !SL Added so the new and old LCC/product use approach both work
!!$       prod_s_total(:) = zero
!!$
!!$       !! 16. Total heterotrophic respiration
!!$
!!$       tot_soil_carb(:,:) = zero
!!$       tot_litter_carb(:,:) = zero
!!$       DO j=1,nvm
!!$
!!$          tot_litter_carb(:,j) = tot_litter_carb(:,j) + (litter(:,istructural,j,iabove,icarbon) + &
!!$               &          litter(:,imetabolic,j,iabove,icarbon) + &
!!$               &          litter(:,istructural,j,ibelow,icarbon) + litter(:,imetabolic,j,ibelow,icarbon) + &
!!$               &          litter(:,iwoody,j,iabove,icarbon) + litter(:,iwoody,j,ibelow,icarbon))
!!$
!!$          tot_soil_carb(:,j) = tot_soil_carb(:,j) + (carbon(:,iactive,j) + &
!!$               &          carbon(:,islow,j)+  carbon(:,ipassive,j))
!!$
!!$       ENDDO
!!$
!!$       tot_litter_soil_carb(:,:) = tot_litter_carb(:,:) + tot_soil_carb(:,:)
!!$
!!$!     DO k = 1, nelements ! Loop over # elements
!!$!        tot_live_biomass(:,:,k) = biomass(:,:,ileaf,k) + biomass(:,:,isapabove,k) + biomass(:,:,isapbelow,k) +&
!!$!             &                    biomass(:,:,iheartabove,k) + biomass(:,:,iheartbelow,k) + &
!!$!             &                    biomass(:,:,iroot,k) + biomass(:,:,ifruit,k) + biomass(:,:,icarbres,k)
!!$!    END DO ! Loop over # elements
!!$
!!$       tot_live_biomass(:,:,:) = biomass(:,:,ileaf,:) + biomass(:,:,isapabove,:) + biomass(:,:,isapbelow,:) +&
!!$            &                    biomass(:,:,iheartabove,:) + biomass(:,:,iheartbelow,:) + &
!!$            &                    biomass(:,:,iroot,:) + biomass(:,:,ifruit,:) + biomass(:,:,icarbres,:)
!!$
!!$       shoot_ratio(:,:) = zero
!!$       WHERE ( (biomass(:,:,isapabove,icarbon) + biomass(:,:,iheartabove,icarbon) & 
!!$            + biomass(:,:,isapbelow,icarbon) + biomass(:,:,iheartbelow,icarbon)) > min_stomate )
!!$
!!$          shoot_ratio(:,:) = (biomass(:,:,isapabove,icarbon) + biomass(:,:,iheartabove,icarbon))/ &
!!$               & (biomass(:,:,isapabove,icarbon) + biomass(:,:,iheartabove,icarbon) + &
!!$               &  biomass(:,:,isapbelow,icarbon) + biomass(:,:,iheartbelow,icarbon))
!!$
!!$       END WHERE
!!$
!!$       woody_biomass_above(:,:) = biomass(:,:,isapabove,icarbon) + biomass(:,:,iheartabove,icarbon) & 
!!$            + biomass(:,:,icarbres,icarbon)* shoot_ratio(:,:)
!!$
!!$       woody_biomass_below(:,:) = biomass(:,:,isapbelow,icarbon) + biomass(:,:,iheartbelow,icarbon) &
!!$            + biomass(:,:,icarbres,icarbon)* (1. - shoot_ratio(:,:))
!!$
!!$       wood_loss_ab(:,:) = exported_bm(:,:) + bm_to_litter(:,:,isapabove,icarbon) & 
!!$            & + bm_to_litter(:,:,iheartabove,icarbon) + &
!!$            & bm_to_litter(:,:,icarbres,icarbon)*shoot_ratio(:,:) + &
!!$            & turnover_daily(:,:,isapabove,icarbon) + turnover_daily(:,:,iheartabove,icarbon) + &
!!$            & turnover_daily(:,:,icarbres,icarbon) * shoot_ratio(:,:)
!!$
!!$       wood_loss_be(:,:) = bm_to_litter(:,:,isapbelow,icarbon) + bm_to_litter(:,:,iheartbelow,icarbon) & 
!!$            & + bm_to_litter(:,:,icarbres,icarbon) * (1. - shoot_ratio(:,:)) &
!!$            & + turnover_daily(:,:,isapbelow,icarbon) + turnover_daily(:,:,iheartbelow,icarbon) &
!!$            & + turnover_daily(:,:,icarbres,icarbon) * (1. - shoot_ratio(:,:))
!!$
!!$       WHERE (.NOT. PFTpresent(:,:))
!!$          woody_biomass_above(:,:) = zero
!!$          woody_biomass_below(:,:) = zero
!!$          wood_loss_ab(:,:) = zero
!!$          wood_loss_be(:,:) = zero
!!$       ENDWHERE
!!$
!!$       tot_turnover(:,:,:) = turnover_daily(:,:,ileaf,:) + turnover_daily(:,:,isapabove,:) + &
!!$            &         turnover_daily(:,:,isapbelow,:) + turnover_daily(:,:,iheartabove,:) + &
!!$            &         turnover_daily(:,:,iheartbelow,:) + turnover_daily(:,:,iroot,:) + &
!!$            &         turnover_daily(:,:,ifruit,:) + turnover_daily(:,:,icarbres,:)
!!$
!!$       tot_bm_to_litter(:,:,:) = bm_to_litter(:,:,ileaf,:) + bm_to_litter(:,:,isapabove,:) + &
!!$            &             bm_to_litter(:,:,isapbelow,:) + bm_to_litter(:,:,iheartbelow,:) + &
!!$            &             bm_to_litter(:,:,iheartabove,:) + bm_to_litter(:,:,iroot,:) + &
!!$            &             bm_to_litter(:,:,ifruit,:) + bm_to_litter(:,:,icarbres,:)
!!$
!!$       carb_mass_variation(:)=-carb_mass_total(:)
!!$       carb_mass_total(:)=SUM((tot_live_biomass(:,:,icarbon)+tot_litter_carb+tot_soil_carb)*veget_max,dim=2) + &
!!$            &                 (prod_m_total + prod_l_total)
!!$       carb_mass_variation(:)=carb_mass_total(:)+carb_mass_variation(:)
!!$
!!$       !! 17. Write history
!!$
!!$       CALL histwrite_p (hist_id_stomate, 'RESOLUTION_X', itime, &
!!$            resolution(:,1), npts, hori_index)
!!$       CALL histwrite_p (hist_id_stomate, 'RESOLUTION_Y', itime, &
!!$            resolution(:,2), npts, hori_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CONTFRAC', itime, &
!!$            contfrac(:), npts, hori_index)
!!$
!!$       CALL histwrite_p (hist_id_stomate, 'LITTER_STR_AB', itime, &
!!$            litter(:,istructural,:,iabove,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'LITTER_MET_AB', itime, &
!!$            litter(:,imetabolic,:,iabove,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'LITTER_STR_BE', itime, &
!!$            litter(:,istructural,:,ibelow,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'LITTER_MET_BE', itime, &
!!$            litter(:,imetabolic,:,ibelow,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'LITTER_WOD_AB', itime, &
!!$            litter(:,iwoody,:,iabove,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'LITTER_WOD_BE', itime, &
!!$            litter(:,iwoody,:,ibelow,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'DEADLEAF_COVER', itime, &
!!$            deadleaf_cover, npts, hori_index)
!!$
!!$       CALL histwrite_p (hist_id_stomate, 'TOTAL_SOIL_CARB', itime, &
!!$            tot_litter_soil_carb, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CARBON_ACTIVE', itime, &
!!$            carbon(:,iactive,:), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CARBON_SLOW', itime, &
!!$            carbon(:,islow,:), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CARBON_PASSIVE', itime, &
!!$            carbon(:,ipassive,:), npts*nvm, horipft_index)
!!$
!!$       CALL histwrite_p (hist_id_stomate, 'T2M_MONTH', itime, &
!!$            t2m_month, npts, hori_index)
!!$       CALL histwrite_p (hist_id_stomate, 'T2M_WEEK', itime, &
!!$            t2m_week, npts, hori_index)
!!$
!!$       CALL histwrite_p (hist_id_stomate, 'HET_RESP', itime, &
!!$            resp_hetero(:,:), npts*nvm, horipft_index)
!!$
!!$       CALL histwrite_p (hist_id_stomate, 'BLACK_CARBON', itime, &
!!$            black_carbon, npts, hori_index)
!!$
!!$       CALL histwrite_p (hist_id_stomate, 'FIREINDEX', itime, &
!!$            fireindex(:,:), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'LITTERHUM', itime, &
!!$            litterhum_daily, npts, hori_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CO2_FIRE', itime, &
!!$            co2_fire, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CO2_TAKEN', itime, &
!!$            co2_to_bm, npts*nvm, horipft_index)
!!$       ! land cover change
!!$       CALL histwrite_p (hist_id_stomate, 'CFLUX_PROD_S', itime, &
!!$            cflux_prod_s, npts, hori_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CFLUX_PROD_M', itime, &
!!$            cflux_prod_m, npts, hori_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CFLUX_PROD_L', itime, &
!!$            cflux_prod_l, npts, hori_index)  
!!$       CALL histwrite_p (hist_id_stomate, 'HARVEST_ABOVE', itime, &
!!$            SUM(harvest_pool(:,:,1,ifm_crop,icut_crop),2), npts, hori_index)
!!$       CALL histwrite_p (hist_id_stomate, 'LAI', itime, &
!!$            lai, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'VEGET_MAX', itime, &
!!$            veget_max, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'NPP', itime, &
!!$            npp_daily, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'GPP', itime, &
!!$            gpp_daily, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'IND', itime, &
!!$            ind, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'CN_IND', itime, &
!!$            cn_ind, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'WOODMASS_IND', itime, &
!!$            woodmass_ind, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'TOTAL_M', itime, &
!!$            tot_live_biomass(:,:,icarbon), npts*nvm, horipft_index)
!!$       !
!!$       IF ( control%forest_management ) THEN
!!$          CALL histwrite_p (hist_id_stomate, 'TOTAL_WOOD_BE', itime, &
!!$               woody_biomass_below(:,:), npts*nvm, horipft_index)
!!$          CALL histwrite_p (hist_id_stomate, 'TOTAL_WOOD_AB', itime, &
!!$               woody_biomass_above(:,:), npts*nvm, horipft_index)
!!$          CALL histwrite_p (hist_id_stomate, 'WOOD_LOSS_AB', itime, &
!!$               wood_loss_ab(:,:), npts*nvm, horipft_index)
!!$          CALL histwrite_p (hist_id_stomate, 'WOOD_LOSS_BE', itime, &
!!$               wood_loss_be(:,:), npts*nvm, horipft_index)
!!$       END IF ! control%forest_management
!!$       !
!!$       CALL histwrite_p (hist_id_stomate, 'LEAF_M', itime, &
!!$            biomass(:,:,ileaf,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'SAP_M_AB', itime, &
!!$            biomass(:,:,isapabove,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'SAP_M_BE', itime, &
!!$            biomass(:,:,isapbelow,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'HEART_M_AB', itime, &
!!$            biomass(:,:,iheartabove,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'HEART_M_BE', itime, &
!!$            biomass(:,:,iheartbelow,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'ROOT_M', itime, &
!!$            biomass(:,:,iroot,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'FRUIT_M', itime, &
!!$            biomass(:,:,ifruit,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'RESERVE_M', itime, &
!!$            biomass(:,:,icarbres,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'TOTAL_TURN', itime, &
!!$            tot_turnover(:,:,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'LEAF_TURN', itime, &
!!$            turnover_daily(:,:,ileaf,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'SAP_AB_TURN', itime, &
!!$            turnover_daily(:,:,isapabove,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'ROOT_TURN', itime, &
!!$            turnover_daily(:,:,iroot,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'FRUIT_TURN', itime, &
!!$            turnover_daily(:,:,ifruit,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'TOTAL_BM_LITTER', itime, &
!!$            tot_bm_to_litter(:,:,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'LEAF_BM_LITTER', itime, &
!!$            bm_to_litter(:,:,ileaf,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'SAP_AB_BM_LITTER', itime, &
!!$            bm_to_litter(:,:,isapabove,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'SAP_BE_BM_LITTER', itime, &
!!$            bm_to_litter(:,:,isapbelow,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'HEART_AB_BM_LITTER', itime, &
!!$            bm_to_litter(:,:,iheartabove,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'HEART_BE_BM_LITTER', itime, &
!!$            bm_to_litter(:,:,iheartbelow,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'ROOT_BM_LITTER', itime, &
!!$            bm_to_litter(:,:,iroot,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'FRUIT_BM_LITTER', itime, &
!!$            bm_to_litter(:,:,ifruit,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'RESERVE_BM_LITTER', itime, &
!!$            bm_to_litter(:,:,icarbres,icarbon), npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'MAINT_RESP', itime, &
!!$            resp_maint, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'GROWTH_RESP', itime, &
!!$            resp_growth, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'AGE', itime, &
!!$            age, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'HEIGHT', itime, &
!!$            height, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'MOISTRESS', itime, &
!!$            moiavail_week, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'VCMAX', itime, &
!!$            vcmax, npts*nvm, horipft_index)
!!$       CALL histwrite_p (hist_id_stomate, 'TURNOVER_TIME', itime, &
!!$            turnover_time, npts*nvm, horipft_index)
!!$       ! land cover change
!!$       CALL histwrite_p (hist_id_stomate, 'PROD_M', itime, &
!!$            prod_m, npts*(nmedium+1), horip_mm_index)
!!$       CALL histwrite_p (hist_id_stomate, 'PROD_L', itime, &
!!$            prod_l, npts*(nlong+1), horip_ll_index)
!!$       CALL histwrite_p (hist_id_stomate, 'FLUX_M', itime, &
!!$            flux_m, npts*nmedium, horip_m_index)
!!$       CALL histwrite_p (hist_id_stomate, 'FLUX_L', itime, &
!!$            flux_l, npts*nlong, horip_l_index)
!!$
!!$       ! forest management
!!$
!!$       ! For state variables, in order to be consistent with other history variables that are
!!$       ! null all the time but the last day of the year, their value is recorded as 0 for all 
!!$       ! days but the last of the year.
!!$
!!$       IF ( control%forest_management ) THEN
!!$
!!$          IF ( EndOfYear ) THEN
!!$             var_real=REAL(age_stand)
!!$             CALL histwrite_p (hist_id_stomate, 'AGE_STAND', itime, &
!!$                  var_real, npts*nvm, horipft_index) 
!!$             var_real=REAL(rotation_n)
!!$             CALL histwrite_p (hist_id_stomate, 'ROTATION_N', itime, &
!!$                  var_real, npts*nvm, horipft_index)
!!$             var_real=REAL(last_cut)
!!$             CALL histwrite_p (hist_id_stomate, 'LAST_CUT', itime, &
!!$                  var_real, npts*nvm, horipft_index)
!!$             var_real=REAL(forest_managed)
!!$             CALL histwrite_p (hist_id_stomate, 'FOREST_MANAGED', itime, &
!!$                  var_real, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'WOOD_LOSS_AB_EOY', itime, &
!!$                  wood_loss_ab, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'EXPORTED_BM', itime, &
!!$                  exported_bm, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'LAI_MAX_LYEAR', itime, &
!!$                  lai_max_lastyear, npts*nvm, horipft_index) 
!!$             lai_max_lastyear(:,:) = zero
!!$             CALL histwrite_p (hist_id_stomate, 'SIGMA', itime, &
!!$                  sigma, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'GAMMA', itime, &
!!$                  gamma, npts*nvm, horipft_index) ! modifTL
!!$             CALL histwrite_p (hist_id_stomate, 'AV_CIRC', itime, &
!!$                  av_circ, npts*nvm, horipft_index) 
!!$             CALL histwrite_p (hist_id_stomate, 'DOM_HEIGHT', itime, &
!!$                  dom_height, npts*nvm, horipft_index) 
!!$             CALL histwrite_p (hist_id_stomate, 'AV_HEIGHT', itime, &
!!$                  av_height, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'MED_HEIGHT', itime, &
!!$                  med_height, npts*nvm, horipft_index) 
!!$             CALL histwrite_p (hist_id_stomate, 'Q1_HEIGHT', itime, &
!!$                  q1_height, npts*nvm, horipft_index) 
!!$             CALL histwrite_p (hist_id_stomate, 'Q3_HEIGHT', itime, &
!!$                  q3_height, npts*nvm, horipft_index) 
!!$             CALL histwrite_p (hist_id_stomate, 'STANDING_AB_BM', itime, &
!!$                  standing_ab_bm, npts*nvm, horipft_index) 
!!$             CALL histwrite_p (hist_id_stomate, 'DELTAVOL', itime, &
!!$                  deltavol, npts*nvm, horipft_index) 
!!$             CALL histwrite_p (hist_id_stomate, 'STANDING_AB_VOL', itime, &
!!$                  standing_ab_vol, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'HAR_VOL', itime, &
!!$                  harvestable_vol, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'HAR_VOL_INC', itime, &
!!$                  harvestable_vol_inc, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'EXPORTED_VOL', itime, &
!!$                  exported_vol, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'EXPORTED_VOL_TH', itime, &
!!$                  exported_vol_th, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CIRC_MIN', itime, &
!!$                  circ_min, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CIRC_MAX', itime, &
!!$                  circ_max, npts*nvm, horipft_index)
!!$!             CALL histwrite_p (hist_id_stomate, 'RDI', itime, &
!!$!                  rdi, npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC02', itime, &
!!$                  circ_class_n(:,:,ic02), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC04', itime, &
!!$                  circ_class_n(:,:,ic04), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC06', itime, &
!!$                  circ_class_n(:,:,ic06), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC08', itime, &
!!$                  circ_class_n(:,:,ic08), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC10', itime, &
!!$                  circ_class_n(:,:,ic10), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC12', itime, &
!!$                  circ_class_n(:,:,ic12), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC14', itime, &
!!$                  circ_class_n(:,:,ic14), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC16', itime, &
!!$                  circ_class_n(:,:,ic16), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC18', itime, &
!!$                  circ_class_n(:,:,ic18), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC20', itime, &
!!$                  circ_class_n(:,:,ic20), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC222', itime, &
!!$                  circ_class_n(:,:,ic222), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CCPO', itime, &
!!$                  circ_class_n(:,:,icpo), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CCSW', itime, &
!!$                  circ_class_n(:,:,icsw), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CCMW', itime, &
!!$                  circ_class_n(:,:,icmw), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CCLW', itime, &
!!$                  circ_class_n(:,:,iclw), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC02_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic02), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC04_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic04), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC06_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic06), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC08_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic08), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC10_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic10), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC12_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic12), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC14_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic14), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC16_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic16), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC18_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic18), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC20_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic20), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CC222_BT', itime, &
!!$                  circ_class_n_bt(:,:,ic222), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CCPO_BT', itime, &
!!$                  circ_class_n_bt(:,:,icpo), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CCSW_BT', itime, &
!!$                  circ_class_n_bt(:,:,icsw), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CCMW_BT', itime, &
!!$                  circ_class_n_bt(:,:,icmw), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'CCLW_BT', itime, &
!!$                  circ_class_n_bt(:,:,iclw), npts*nvm, horipft_index)
!!$             CALL histwrite_p (hist_id_stomate, 'IND_BT', itime, &
!!$                  ind_bt, npts*nvm, horipft_index)
!!$          ENDIF
!!$
!!$       END IF  ! control%forest_management
!!$
!!$       IF ( hist_id_stomate_IPCC > 0 ) THEN
!!$          vartmp(:)=SUM(tot_live_biomass(:,:,icarbon)*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cVeg", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(tot_litter_carb*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cLitter", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(tot_soil_carb*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cSoil", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=(prod_m_total + prod_l_total)/1e3
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cProduct", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=carb_mass_variation/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cMassVariation", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(lai*veget_max,dim=2)*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "lai", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(gpp_daily*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "gpp", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM((resp_maint+resp_growth)*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "ra", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(npp_daily*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "npp", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(resp_hetero*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "rh", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(co2_fire*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "fFire", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(harvest_pool(:,:,1,ifm_crop,icut_crop),2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "fHarvest", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=cflux_prod_total/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "fLuc", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=(SUM((gpp_daily-(resp_maint+resp_growth+resp_hetero)-co2_fire) * &
!!$               veget_max,dim=2)-cflux_prod_total-SUM(harvest_pool(:,:,1,ifm_crop,icut_crop),2))/ &
!!$               1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "nbp", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM((tot_bm_to_litter(:,:,icarbon) + tot_turnover(:,:,icarbon))*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "fVegLitter", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(SUM(soilcarbon_input,dim=2)*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "fLitterSoil", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(biomass(:,:,ileaf,icarbon)*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cLeaf", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM((biomass(:,:,isapabove,icarbon)+biomass(:,:,iheartabove,icarbon))*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cWood", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(( biomass(:,:,iroot,icarbon) + biomass(:,:,isapbelow,icarbon) + biomass(:,:,iheartbelow,icarbon) ) &
!!$               &        *veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cRoot", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(( biomass(:,:,icarbres,icarbon) + biomass(:,:,ifruit,icarbon))*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cMisc", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM((litter(:,istructural,:,iabove,icarbon)+litter(:,imetabolic,:,iabove,icarbon))*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cLitterAbove", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM((litter(:,istructural,:,ibelow,icarbon)+litter(:,imetabolic,:,ibelow,icarbon))*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cLitterBelow", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(carbon(:,iactive,:)*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cSoilFast", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(carbon(:,islow,:)*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cSoilMedium", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(carbon(:,ipassive,:)*veget_max,dim=2)/1e3*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "cSoilSlow", itime, &
!!$               vartmp, npts, hori_index)
!!$          DO j=1,nvm
!!$             histvar(:,j)=veget_max(:,j)*contfrac(:)*100
!!$          ENDDO
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "landCoverFrac", itime, &
!!$               histvar, npts*nvm, horipft_index)
!!$          !-
!!$          vartmp(:)=zero
!!$          DO j = 2,nvm
!!$             IF (is_deciduous(j)) THEN
!!$                vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
!!$             ENDIF
!!$          ENDDO
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "treeFracPrimDec", itime, &
!!$               vartmp, npts, hori_index)
!!$          !-
!!$          vartmp(:)=zero
!!$          DO j = 2,nvm
!!$             IF (is_evergreen(j)) THEN
!!$                vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
!!$             ENDIF
!!$          ENDDO
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "treeFracPrimEver", itime, &
!!$               vartmp, npts, hori_index)
!!$          !-
!!$          vartmp(:)=zero
!!$          DO j = 2,nvm
!!$             IF ( .NOT.(is_c4(j)) ) THEN
!!$                vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
!!$             ENDIF
!!$          ENDDO
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "c3PftFrac", itime, &
!!$               vartmp, npts, hori_index)
!!$          !-
!!$          vartmp(:)=zero
!!$          DO j = 2,nvm
!!$             IF ( is_c4(j) ) THEN
!!$                vartmp(:) = vartmp(:) + veget_max(:,j)*contfrac*100
!!$             ENDIF
!!$          ENDDO
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "c4PftFrac", itime, &
!!$               vartmp, npts, hori_index)
!!$          !-
!!$          vartmp(:)=SUM(resp_growth*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "rGrowth", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(resp_maint*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "rMaint", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(bm_alloc(:,:,ileaf,icarbon)*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "nppLeaf", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(bm_alloc(:,:,isapabove,icarbon)*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "nppWood", itime, &
!!$               vartmp, npts, hori_index)
!!$          vartmp(:)=SUM(( bm_alloc(:,:,isapbelow,icarbon) + bm_alloc(:,:,iroot,icarbon) )*veget_max,dim=2)/1e3/one_day*contfrac
!!$          CALL histwrite_p (hist_id_stomate_IPCC, "nppRoot", itime, &
!!$               vartmp, npts, hori_index)
!!$
!!$          CALL histwrite_p (hist_id_stomate_IPCC, 'RESOLUTION_X', itime, &
!!$               resolution(:,1), npts, hori_index)
!!$          CALL histwrite_p (hist_id_stomate_IPCC, 'RESOLUTION_Y', itime, &
!!$               resolution(:,2), npts, hori_index)
!!$          CALL histwrite_p (hist_id_stomate_IPCC, 'CONTFRAC', itime, &
!!$               contfrac(:), npts, hori_index)
!!$
!!$       ENDIF

    ENDIF ! checking to see if we are doing new or old allocation

    !---TEMP---
    CALL check_biomass_sync("stomate_lpj.f90: After LPJ", npts, biomass, &
         circ_class_biomass, circ_class_n, ind, &
         lsync, bm_sync)

    firstcall = .FALSE.

    IF (bavard.GE.2) WRITE(numout,*) 'Leaving stomate_lpj'

  END SUBROUTINE StomateLpj

END MODULE stomate_lpj
